@@ -20,6 +20,7 @@
 #include "nsIPresShell.h"
 #include "nsFocusManager.h"
 #include "nsEventDispatcher.h"
+#include "nsDOMDataTransfer.h"
 
 #include "nsIDocShell.h"
 #include "nsIContentViewerEdit.h"
@@ -78,34 +79,21 @@ SelectionCopyHelper(nsISelection *aSel, nsIDocument *aDoc,
     *aTransferable = nullptr;
   }
 
-  nsresult rv = NS_OK;
-  
-  bool bIsPlainTextContext = false;
-
-  rv = nsCopySupport::IsPlainTextContext(aSel, aDoc, &bIsPlainTextContext);
-  if (NS_FAILED(rv)) 
-    return rv;
-
-  bool bIsHTMLCopy = !bIsPlainTextContext;
-  nsAutoString mimeType;
+  nsresult rv;
 
   nsCOMPtr<nsIDocumentEncoder> docEncoder;
-
   docEncoder = do_CreateInstance(NS_HTMLCOPY_ENCODER_CONTRACTID);
   NS_ENSURE_TRUE(docEncoder, NS_ERROR_FAILURE);
 
-  // We always require a plaintext version
-  
   // note that we assign text/unicode as mime type, but in fact nsHTMLCopyEncoder
   // ignore it and use text/html or text/plain depending where the selection
   // is. if it is a selection into input/textarea element or in a html content
   // with pre-wrap style : text/plain. Otherwise text/html.
   // see nsHTMLCopyEncoder::SetSelection
+  nsAutoString mimeType;
   mimeType.AssignLiteral(kUnicodeMime);
-  
-  // we want preformatted for the case where the selection is inside input/textarea
-  // and we don't want pretty printing for others cases, to not have additionnal
-  // line breaks which are then converted into spaces by the htmlConverter (see bug #524975)
+
+  // Do the first and potentially trial encoding as preformatted and raw.
   uint32_t flags = aFlags | nsIDocumentEncoder::OutputPreformatted
                           | nsIDocumentEncoder::OutputRaw;
 
@@ -113,28 +101,45 @@ SelectionCopyHelper(nsISelection *aSel, nsIDocument *aDoc,
   NS_ASSERTION(domDoc, "Need a document");
 
   rv = docEncoder->Init(domDoc, mimeType, flags);
-  if (NS_FAILED(rv)) 
-    return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = docEncoder->SetSelection(aSel);
-  if (NS_FAILED(rv)) 
-    return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsAutoString buffer, parents, info, textBuffer, plaintextBuffer;
+  // SetSelection set the mime type to text/plain if the selection is inside a
+  // text widget.
+  rv = docEncoder->GetMimeType(mimeType);
+  NS_ENSURE_SUCCESS(rv, rv);
+  bool selForcedTextPlain = mimeType.EqualsLiteral(kTextMime);
 
-  rv = docEncoder->EncodeToString(textBuffer);
-  if (NS_FAILED(rv)) 
-    return rv;
+  nsAutoString buf;
+  rv = docEncoder->EncodeToString(buf);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  // If the selection was in a text input, in textarea or in pre, the encoder
-  // already produced plain text. Otherwise,the encoder produced HTML. In that
-  // case, we need to create an additional plain text serialization and an
-  // addition HTML serialization that encodes context.
-  if (bIsHTMLCopy) {
+  rv = docEncoder->GetMimeType(mimeType);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    // First, create the plain text serialization
-    mimeType.AssignLiteral("text/plain");
+  if (!selForcedTextPlain && mimeType.EqualsLiteral(kTextMime)) {
+    // SetSelection and EncodeToString use this case to signal that text/plain
+    // was forced because the document is either not an nsIHTMLDocument or it's
+    // XHTML.  We want to pretty print XHTML but not non-nsIHTMLDocuments.
+    nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(aDoc);
+    if (!htmlDoc) {
+      selForcedTextPlain = true;
+    }
+  }
 
+  // The mime type is ultimately text/html if the encoder successfully encoded
+  // the selection as text/html.
+  bool encodedTextHTML = mimeType.EqualsLiteral(kHTMLMime);
+
+  // First, prepare the text/plain clipboard flavor.
+  nsAutoString textPlainBuf;
+  if (selForcedTextPlain) {
+    // Nothing to do.  buf contains the final, preformatted, raw text/plain.
+    textPlainBuf.Assign(buf);
+  } else {
+    // Redo the encoding, but this time use pretty printing.
     flags =
       nsIDocumentEncoder::OutputSelectionOnly |
       nsIDocumentEncoder::OutputAbsoluteLinks |
@@ -142,35 +147,35 @@ SelectionCopyHelper(nsISelection *aSel, nsIDocument *aDoc,
       nsIDocumentEncoder::OutputDropInvisibleBreak |
       (aFlags & nsIDocumentEncoder::OutputNoScriptContent);
 
-    rv = docEncoder->Init(domDoc, mimeType, flags);
-    if (NS_FAILED(rv))
-      return rv;
-
-    rv = docEncoder->SetSelection(aSel);
-    if (NS_FAILED(rv))
-      return rv;
-
-    rv = docEncoder->EncodeToString(plaintextBuffer);
-    if (NS_FAILED(rv))
-      return rv;
-
-    // Now create the version that shows HTML context
-
-    mimeType.AssignLiteral(kHTMLMime);
-
-    flags = aFlags;
-
+    mimeType.AssignLiteral(kTextMime);
     rv = docEncoder->Init(domDoc, mimeType, flags);
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = docEncoder->SetSelection(aSel);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    // encode the selection as html with contextual info
-    rv = docEncoder->EncodeToStringWithContext(parents, info, buffer);
+    rv = docEncoder->EncodeToString(textPlainBuf);
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  
+
+  // Second, prepare the text/html flavor.
+  nsAutoString textHTMLBuf;
+  nsAutoString htmlParentsBuf;
+  nsAutoString htmlInfoBuf;
+  if (encodedTextHTML) {
+    // Redo the encoding, but this time use the passed-in flags.
+    mimeType.AssignLiteral(kHTMLMime);
+    rv = docEncoder->Init(domDoc, mimeType, aFlags);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = docEncoder->SetSelection(aSel);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = docEncoder->EncodeToStringWithContext(htmlParentsBuf, htmlInfoBuf,
+                                               textHTMLBuf);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   // Get the Clipboard
   nsCOMPtr<nsIClipboard> clipboard;
   if (doPutOnClipboard) {
@@ -184,37 +189,37 @@ SelectionCopyHelper(nsISelection *aSel, nsIDocument *aDoc,
     nsCOMPtr<nsITransferable> trans = do_CreateInstance(kCTransferableCID);
     if (trans) {
       trans->Init(aDoc->GetLoadContext());
-      if (bIsHTMLCopy) {
+      if (encodedTextHTML) {
         // Set up a format converter so that clipboard flavor queries work.
         // This converter isn't really used for conversions.
         nsCOMPtr<nsIFormatConverter> htmlConverter =
           do_CreateInstance(kHTMLConverterCID);
         trans->SetConverter(htmlConverter);
 
-        if (!buffer.IsEmpty()) {
+        if (!textHTMLBuf.IsEmpty()) {
           // Add the html DataFlavor to the transferable
-          rv = AppendString(trans, buffer, kHTMLMime);
+          rv = AppendString(trans, textHTMLBuf, kHTMLMime);
           NS_ENSURE_SUCCESS(rv, rv);
         }
 
         // Add the htmlcontext DataFlavor to the transferable
         // Even if parents is empty string, this flavor should
         // be attached to the transferable
-        rv = AppendString(trans, parents, kHTMLContext);
+        rv = AppendString(trans, htmlParentsBuf, kHTMLContext);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        if (!info.IsEmpty()) {
+        if (!htmlInfoBuf.IsEmpty()) {
           // Add the htmlinfo DataFlavor to the transferable
-          rv = AppendString(trans, info, kHTMLInfo);
+          rv = AppendString(trans, htmlInfoBuf, kHTMLInfo);
           NS_ENSURE_SUCCESS(rv, rv);
         }
 
-        if (!plaintextBuffer.IsEmpty()) {
+        if (!textPlainBuf.IsEmpty()) {
           // unicode text
           // Add the unicode DataFlavor to the transferable
           // If we didn't have this, then nsDataObj::GetData matches text/unicode against
           // the kURLMime flavour which is not desirable (eg. when pasting into Notepad)
-          rv = AppendString(trans, plaintextBuffer, kUnicodeMime);
+          rv = AppendString(trans, textPlainBuf, kUnicodeMime);
           NS_ENSURE_SUCCESS(rv, rv);
         }
 
@@ -238,9 +243,9 @@ SelectionCopyHelper(nsISelection *aSel, nsIDocument *aDoc,
           }
         }
       } else {
-        if (!textBuffer.IsEmpty()) {
+        if (!textPlainBuf.IsEmpty()) {
           // Add the unicode DataFlavor to the transferable
-          rv = AppendString(trans, textBuffer, kUnicodeMime);
+          rv = AppendString(trans, textPlainBuf, kUnicodeMime);
           NS_ENSURE_SUCCESS(rv, rv);
         }
       }
@@ -293,7 +298,7 @@ nsCopySupport::GetTransferableForNode(nsINode* aNode,
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aNode);
   NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
-  nsRefPtr<nsRange> range = new nsRange();
+  nsRefPtr<nsRange> range = new nsRange(aNode);
   rv = range->SelectNode(node);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = selection->AddRange(range);
@@ -347,81 +352,6 @@ nsresult nsCopySupport::DoHooks(nsIDocument *aDoc, nsITransferable *aTrans,
   return rv;
 }
 
-nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc, bool *aIsPlainTextContext)
-{
-  nsresult rv;
-
-  if (!aSel || !aIsPlainTextContext)
-    return NS_ERROR_NULL_POINTER;
-
-  *aIsPlainTextContext = false;
-  
-  nsCOMPtr<nsIDOMRange> range;
-  nsCOMPtr<nsIDOMNode> commonParent;
-  int32_t count = 0;
-
-  rv = aSel->GetRangeCount(&count);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // if selection is uninitialized return
-  if (!count)
-    return NS_ERROR_FAILURE;
-  
-  // we'll just use the common parent of the first range.  Implicit assumption
-  // here that multi-range selections are table cell selections, in which case
-  // the common parent is somewhere in the table and we don't really care where.
-  rv = aSel->GetRangeAt(0, getter_AddRefs(range));
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (!range)
-    return NS_ERROR_NULL_POINTER;
-  range->GetCommonAncestorContainer(getter_AddRefs(commonParent));
-
-  for (nsCOMPtr<nsIContent> selContent(do_QueryInterface(commonParent));
-       selContent;
-       selContent = selContent->GetParent())
-  {
-    // checking for selection inside a plaintext form widget
-
-    if (!selContent->IsHTML()) {
-      continue;
-    }
-
-    nsIAtom *atom = selContent->Tag();
-
-    if (atom == nsGkAtoms::input ||
-        atom == nsGkAtoms::textarea)
-    {
-      *aIsPlainTextContext = true;
-      break;
-    }
-
-    if (atom == nsGkAtoms::body)
-    {
-      // check for moz prewrap style on body.  If it's there we are 
-      // in a plaintext editor.  This is pretty cheezy but I haven't 
-      // found a good way to tell if we are in a plaintext editor.
-      nsCOMPtr<nsIDOMElement> bodyElem = do_QueryInterface(selContent);
-      nsAutoString wsVal;
-      rv = bodyElem->GetAttribute(NS_LITERAL_STRING("style"), wsVal);
-      if (NS_SUCCEEDED(rv) && (kNotFound != wsVal.Find(NS_LITERAL_STRING("pre-wrap"))))
-      {
-        *aIsPlainTextContext = true;
-        break;
-      }
-    }
-  }
-  
-  // also consider ourselves in a text widget if we can't find an html
-  // document. Note that XHTML is not counted as HTML here, because we can't
-  // copy it properly (all the copy code for non-plaintext assumes using HTML
-  // serializers and parsers is OK, and those mess up XHTML).
-  nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(aDoc);
-  if (!(htmlDoc && aDoc->IsHTML()))
-    *aIsPlainTextContext = true;
-
-  return NS_OK;
-}
-
 nsresult
 nsCopySupport::GetContents(const nsACString& aMimeType, uint32_t aFlags, nsISelection *aSel, nsIDocument *aDoc, nsAString& outdata)
 {
@@ -678,6 +608,7 @@ nsCopySupport::FireClipboardEvent(int32_t aType, nsIPresShell* aPresShell, nsISe
     content = GetSelectionForCopy(doc, getter_AddRefs(sel));
 
   // retrieve the event target node from the start of the selection
+  nsresult rv;
   if (sel) {
     // Only cut or copy when there is an uncollapsed selection
     if (aType == NS_CUT || aType == NS_COPY) {
@@ -688,7 +619,7 @@ nsCopySupport::FireClipboardEvent(int32_t aType, nsIPresShell* aPresShell, nsISe
     }
 
     nsCOMPtr<nsIDOMRange> range;
-    nsresult rv = sel->GetRangeAt(0, getter_AddRefs(range));
+    rv = sel->GetRangeAt(0, getter_AddRefs(range));
     if (NS_SUCCEEDED(rv) && range) {
       nsCOMPtr<nsIDOMNode> startContainer;
       range->GetStartContainer(getter_AddRefs(startContainer));
@@ -708,25 +639,35 @@ nsCopySupport::FireClipboardEvent(int32_t aType, nsIPresShell* aPresShell, nsISe
   if (!nsContentUtils::IsSafeToRunScript())
     return false;
 
-  // next, fire the cut or copy event
+  // next, fire the cut, copy or paste event
+  // XXXndeakin Bug 844941 - why was a preference added here without a running-in-chrome check?
+  bool doDefault = true;
+  nsRefPtr<nsDOMDataTransfer> clipboardData;
   if (Preferences::GetBool("dom.event.clipboardevents.enabled", true)) {
+    clipboardData = new nsDOMDataTransfer(aType, aType == NS_PASTE);
+
     nsEventStatus status = nsEventStatus_eIgnore;
-    nsEvent evt(true, aType);
+    nsClipboardEvent evt(true, aType);
+    evt.clipboardData = clipboardData;
     nsEventDispatcher::Dispatch(content, presShell->GetPresContext(), &evt, nullptr,
                                 &status);
-    // if the event was cancelled, don't do the clipboard operation
-    if (status == nsEventStatus_eConsumeNoDefault)
-      return false;
+    // If the event was cancelled, don't do the clipboard operation
+    doDefault = (status != nsEventStatus_eConsumeNoDefault);
   }
   
-  if (presShell->IsDestroying())
-    return false;
-
   // No need to do anything special during a paste. Either an event listener
   // took care of it and cancelled the event, or the caller will handle it.
-  // Return true to indicate the event wasn't cancelled.
-  if (aType == NS_PASTE)
-    return true;
+  // Return true to indicate that the event wasn't cancelled.
+  if (aType == NS_PASTE) {
+    // Clear and mark the clipboardData as readonly. This prevents someone
+    // from reading the clipboard contents after the paste event has fired.
+    if (clipboardData) {
+      clipboardData->ClearAll();
+      clipboardData->SetReadOnly();
+    }
+
+    return doDefault;
+  }
 
   // Update the presentation in case the event handler modified the selection,
   // see bug 602231.
@@ -734,13 +675,46 @@ nsCopySupport::FireClipboardEvent(int32_t aType, nsIPresShell* aPresShell, nsISe
   if (presShell->IsDestroying())
     return false;
 
-  // call the copy code
-  if (NS_FAILED(nsCopySupport::HTMLCopy(sel, doc, nsIClipboard::kGlobalClipboard)))
-    return false;
+  // if the event was not cancelled, do the default copy. If the event was cancelled,
+  // use the data added to the data transfer and copy that instead.
+  uint32_t count = 0;
+  if (doDefault) {
+    // get the data from the selection if any
+    bool isCollapsed;
+    sel->GetIsCollapsed(&isCollapsed);
+    if (isCollapsed) {
+      return false;
+    }
+    // call the copy code
+    rv = HTMLCopy(sel, doc, nsIClipboard::kGlobalClipboard);
+    if (NS_FAILED(rv)) {
+      return false;
+    }
+  } else if (clipboardData) {
+    // check to see if any data was put on the data transfer.
+    clipboardData->GetMozItemCount(&count);
+    if (count) {
+      nsCOMPtr<nsIClipboard> clipboard(do_GetService("@mozilla.org/widget/clipboard;1"));
+      NS_ENSURE_TRUE(clipboard, false);
+
+      nsCOMPtr<nsITransferable> transferable =
+        clipboardData->GetTransferable(0, doc->GetLoadContext());
+
+      NS_ENSURE_TRUE(transferable, false);
+
+      // put the transferable on the clipboard
+      rv = clipboard->SetData(transferable, nullptr, nsIClipboard::kGlobalClipboard);
+      if (NS_FAILED(rv)) {
+        return false;
+      }
+    }
+  }
 
   // Now that we have copied, update the clipboard commands. This should have
-  // the effect of updating the paste menu item.
-  piWindow->UpdateCommands(NS_LITERAL_STRING("clipboard"));
+  // the effect of updating the enabled state of the paste menu item.
+  if (doDefault || count) {
+    piWindow->UpdateCommands(NS_LITERAL_STRING("clipboard"));
+  }
 
-  return true;
+  return doDefault;
 }
