@@ -7,23 +7,27 @@
 #include "base/basictypes.h"
 
 #include "IDBFactory.h"
+
 #include "nsIFile.h"
-#include "nsIJSContextStack.h"
 #include "nsIPrincipal.h"
 #include "nsIScriptContext.h"
 #include "nsIXPConnect.h"
 #include "nsIXPCScriptable.h"
 
-#include "jsdbgapi.h"
+#include <algorithm>
 #include "mozilla/dom/ContentParent.h"
 #include "mozilla/dom/ContentChild.h"
+#include "mozilla/dom/IDBFactoryBinding.h"
 #include "mozilla/dom/PBrowserChild.h"
+#include "mozilla/dom/quota/OriginOrPatternString.h"
+#include "mozilla/dom/quota/QuotaManager.h"
 #include "mozilla/dom/TabChild.h"
 #include "mozilla/storage.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsCharSeparatedTokenizer.h"
 #include "nsContentUtils.h"
+#include "nsCxPusher.h"
 #include "nsDOMClassInfoID.h"
 #include "nsGlobalWindow.h"
 #include "nsHashKeys.h"
@@ -40,15 +44,19 @@
 #include "IDBKeyRange.h"
 #include "IndexedDatabaseManager.h"
 #include "Key.h"
+#include "ProfilerHelpers.h"
 
 #include "ipc/IndexedDBChild.h"
-#include <algorithm>
 
 USING_INDEXEDDB_NAMESPACE
+USING_QUOTA_NAMESPACE
 
 using mozilla::dom::ContentChild;
 using mozilla::dom::ContentParent;
+using mozilla::dom::NonNull;
+using mozilla::dom::Optional;
 using mozilla::dom::TabChild;
+using mozilla::ErrorResult;
 
 namespace {
 
@@ -67,6 +75,7 @@ IDBFactory::IDBFactory()
 : mOwningObject(nullptr), mActorChild(nullptr), mActorParent(nullptr),
   mContentParent(nullptr), mRootedOwningObject(false)
 {
+  SetIsDOMBinding();
 }
 
 IDBFactory::~IDBFactory()
@@ -103,7 +112,7 @@ IDBFactory::Create(nsPIDOMWindow* aWindow,
 
   // Make sure that the manager is up before we do anything here since lots of
   // decisions depend on which process we're running in.
-  nsRefPtr<indexedDB::IndexedDatabaseManager> mgr =
+  indexedDB::IndexedDatabaseManager* mgr =
     indexedDB::IndexedDatabaseManager::GetOrCreate();
   NS_ENSURE_TRUE(mgr, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
@@ -111,7 +120,7 @@ IDBFactory::Create(nsPIDOMWindow* aWindow,
 
   nsCString origin(aASCIIOrigin);
   if (origin.IsEmpty()) {
-    rv = IndexedDatabaseManager::GetASCIIOriginFromWindow(aWindow, origin);
+    rv = QuotaManager::GetASCIIOriginFromWindow(aWindow, origin);
     if (NS_FAILED(rv)) {
       // Not allowed.
       *aFactory = nullptr;
@@ -149,7 +158,7 @@ IDBFactory::Create(nsPIDOMWindow* aWindow,
 // static
 nsresult
 IDBFactory::Create(JSContext* aCx,
-                   JSObject* aOwningObject,
+                   JS::Handle<JSObject*> aOwningObject,
                    ContentParent* aContentParent,
                    IDBFactory** aFactory)
 {
@@ -161,8 +170,7 @@ IDBFactory::Create(JSContext* aCx,
   NS_ASSERTION(nsContentUtils::IsCallerChrome(), "Only for chrome!");
 
   nsCString origin;
-  nsresult rv =
-    IndexedDatabaseManager::GetASCIIOriginFromWindow(nullptr, origin);
+  nsresult rv = QuotaManager::GetASCIIOriginFromWindow(nullptr, origin);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   nsRefPtr<IDBFactory> factory = new IDBFactory();
@@ -195,35 +203,13 @@ IDBFactory::Create(ContentParent* aContentParent,
   NS_ASSERTION(nsContentUtils::IsCallerChrome(), "Only for chrome!");
   NS_ASSERTION(aContentParent, "Null ContentParent!");
 
-#ifdef DEBUG
-  {
-    nsIThreadJSContextStack* cxStack = nsContentUtils::ThreadJSContextStack();
-    NS_ASSERTION(cxStack, "Couldn't get ThreadJSContextStack!");
-
-    JSContext* lastCx;
-    if (NS_SUCCEEDED(cxStack->Peek(&lastCx))) {
-      NS_ASSERTION(!lastCx, "We should only be called from C++!");
-    }
-    else {
-      NS_ERROR("nsIThreadJSContextStack::Peek should never fail!");
-    }
-  }
-#endif
+  NS_ASSERTION(!nsContentUtils::GetCurrentJSContext(), "Should be called from C++");
 
   nsCOMPtr<nsIPrincipal> principal =
     do_CreateInstance("@mozilla.org/nullprincipal;1");
   NS_ENSURE_TRUE(principal, NS_ERROR_FAILURE);
 
-  JSContext* cx = nsContentUtils::GetSafeJSContext();
-  NS_ENSURE_TRUE(cx, NS_ERROR_FAILURE);
-
-  nsCxPusher pusher;
-  if (!pusher.Push(cx)) {
-    NS_WARNING("Failed to push safe JS context!");
-    return NS_ERROR_FAILURE;
-  }
-
-  JSAutoRequest ar(cx);
+  AutoSafeJSContext cx;
 
   nsIXPConnect* xpc = nsContentUtils::XPConnect();
   NS_ASSERTION(xpc, "This should never be null!");
@@ -232,13 +218,12 @@ IDBFactory::Create(ContentParent* aContentParent,
   nsresult rv = xpc->CreateSandbox(cx, principal, getter_AddRefs(globalHolder));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  JSObject* global;
-  rv = globalHolder->GetJSObject(&global);
-  NS_ENSURE_SUCCESS(rv, rv);
+  JS::Rooted<JSObject*> global(cx, globalHolder->GetJSObject());
+  NS_ENSURE_STATE(global);
 
   // The CreateSandbox call returns a proxy to the actual sandbox object. We
   // don't need a proxy here.
-  global = JS_UnwrapObject(global);
+  global = js::UncheckedUnwrap(global);
 
   JSAutoCompartment ac(cx, global);
 
@@ -301,21 +286,40 @@ IDBFactory::GetConnection(const nsAString& aDatabaseFilePath,
   rv = ss->OpenDatabaseWithFileURL(dbFileUrl, getter_AddRefs(connection));
   NS_ENSURE_SUCCESS(rv, nullptr);
 
-  // Turn on foreign key constraints and recursive triggers.
-  // The "INSERT OR REPLACE" statement doesn't fire the update trigger,
-  // instead it fires only the insert trigger. This confuses the update
-  // refcount function. This behavior changes with enabled recursive triggers,
-  // so the statement fires the delete trigger first and then the insert
-  // trigger.
-  rv = connection->ExecuteSimpleSQL(NS_LITERAL_CSTRING(
-    "PRAGMA foreign_keys = ON; "
-    "PRAGMA recursive_triggers = ON;"
-  ));
+  rv = SetDefaultPragmas(connection);
   NS_ENSURE_SUCCESS(rv, nullptr);
 
   return connection.forget();
 }
 
+// static
+nsresult
+IDBFactory::SetDefaultPragmas(mozIStorageConnection* aConnection)
+{
+  NS_ASSERTION(aConnection, "Null connection!");
+
+  static const char query[] =
+#if defined(MOZ_WIDGET_ANDROID) || defined(MOZ_WIDGET_GONK)
+    // Switch the journaling mode to TRUNCATE to avoid changing the directory
+    // structure at the conclusion of every transaction for devices with slower
+    // file systems.
+    "PRAGMA journal_mode = TRUNCATE; "
+#endif
+    // We use foreign keys in lots of places.
+    "PRAGMA foreign_keys = ON; "
+    // The "INSERT OR REPLACE" statement doesn't fire the update trigger,
+    // instead it fires only the insert trigger. This confuses the update
+    // refcount function. This behavior changes with enabled recursive triggers,
+    // so the statement fires the delete trigger first and then the insert
+    // trigger.
+    "PRAGMA recursive_triggers = ON;";
+
+  nsresult rv = aConnection->ExecuteSimpleSQL(NS_LITERAL_CSTRING(query));
+  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+
+  return NS_OK;
+}
+
 inline
 bool
 IgnoreWhitespace(PRUnichar c)
@@ -330,7 +334,7 @@ IDBFactory::LoadDatabaseInformation(mozIStorageConnection* aConnection,
                                     uint64_t* aVersion,
                                     ObjectStoreInfoArray& aObjectStores)
 {
-  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+  AssertIsOnIOThread();
   NS_ASSERTION(aConnection, "Null pointer!");
 
   aObjectStores.Clear();
@@ -489,9 +493,8 @@ NS_IMPL_CYCLE_COLLECTING_ADDREF(IDBFactory)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(IDBFactory)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(IDBFactory)
+  NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_INTERFACE_MAP_ENTRY(nsIIDBFactory)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(IDBFactory)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(IDBFactory)
@@ -500,6 +503,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(IDBFactory)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(IDBFactory)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
   if (tmp->mOwningObject) {
     tmp->mOwningObject = nullptr;
   }
@@ -511,25 +515,26 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(IDBFactory)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(IDBFactory)
+  NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER
   NS_IMPL_CYCLE_COLLECTION_TRACE_JS_MEMBER_CALLBACK(mOwningObject)
 NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 DOMCI_DATA(IDBFactory, IDBFactory)
 
 nsresult
-IDBFactory::OpenCommon(const nsAString& aName,
-                       int64_t aVersion,
-                       const nsACString& aASCIIOrigin,
-                       bool aDeleting,
-                       JSContext* aCallingCx,
-                       IDBOpenDBRequest** _retval)
+IDBFactory::OpenInternal(const nsAString& aName,
+                         int64_t aVersion,
+                         const nsACString& aASCIIOrigin,
+                         bool aDeleting,
+                         IDBOpenDBRequest** _retval)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(mWindow || mOwningObject, "Must have one of these!");
 
+  AutoJSContext cx;
   nsCOMPtr<nsPIDOMWindow> window;
-  JSObject* scriptOwner = nullptr;
-  FactoryPrivilege privilege;
+  JS::Rooted<JSObject*> scriptOwner(cx);
+  StoragePrivilege privilege;
 
   if (mWindow) {
     window = mWindow;
@@ -543,7 +548,7 @@ IDBFactory::OpenCommon(const nsAString& aName,
   }
 
   nsRefPtr<IDBOpenDBRequest> request =
-    IDBOpenDBRequest::Create(this, window, scriptOwner, aCallingCx);
+    IDBOpenDBRequest::Create(this, window, scriptOwner);
   NS_ENSURE_TRUE(request, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   nsresult rv;
@@ -559,17 +564,17 @@ IDBFactory::OpenCommon(const nsAString& aName,
     nsRefPtr<CheckPermissionsHelper> permissionHelper =
       new CheckPermissionsHelper(openHelper, window, aDeleting);
 
-    IndexedDatabaseManager* mgr = IndexedDatabaseManager::Get();
-    NS_ASSERTION(mgr, "This should never be null!");
+    QuotaManager* quotaManager = QuotaManager::Get();
+    NS_ASSERTION(quotaManager, "This should never be null!");
 
-    rv =
-      mgr->WaitForOpenAllowed(OriginOrPatternString::FromOrigin(aASCIIOrigin),
-                              openHelper->Id(), permissionHelper);
+    rv = quotaManager->WaitForOpenAllowed(OriginOrPatternString::FromOrigin(
+                                          aASCIIOrigin), openHelper->Id(),
+                                          permissionHelper);
     NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
   }
   else if (aDeleting) {
     nsCOMPtr<nsIAtom> databaseId =
-      IndexedDatabaseManager::GetDatabaseId(aASCIIOrigin, aName);
+      QuotaManager::GetStorageId(aASCIIOrigin, aName);
     NS_ENSURE_TRUE(databaseId, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
     IndexedDBDeleteDatabaseRequestChild* actor =
@@ -588,64 +593,123 @@ IDBFactory::OpenCommon(const nsAString& aName,
     dbActor->SetRequest(request);
   }
 
-  request.forget(_retval);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-IDBFactory::Open(const nsAString& aName,
-                 int64_t aVersion,
-                 JSContext* aCx,
-                 uint8_t aArgc,
-                 nsIIDBOpenDBRequest** _retval)
-{
-  if (aVersion < 1 && aArgc) {
-    return NS_ERROR_TYPE_ERR;
+#ifdef IDB_PROFILER_USE_MARKS
+  {
+    NS_ConvertUTF16toUTF8 profilerName(aName);
+    if (aDeleting) {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: deleteDatabase(\"%s\")",
+                        "MT IDBFactory.deleteDatabase()",
+                        request->GetSerialNumber(), profilerName.get());
+    }
+    else {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: open(\"%s\", %lld)",
+                        "MT IDBFactory.open()",
+                        request->GetSerialNumber(), profilerName.get(),
+                        aVersion);
+    }
   }
-
-  nsRefPtr<IDBOpenDBRequest> request;
-  nsresult rv = OpenCommon(aName, aVersion, false, aCx,
-                           getter_AddRefs(request));
-  NS_ENSURE_SUCCESS(rv, rv);
+#endif
 
   request.forget(_retval);
   return NS_OK;
 }
 
-NS_IMETHODIMP
-IDBFactory::DeleteDatabase(const nsAString& aName,
-                           JSContext* aCx,
-                           nsIIDBOpenDBRequest** _retval)
+JSObject*
+IDBFactory::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
 {
-  nsRefPtr<IDBOpenDBRequest> request;
-  nsresult rv = OpenCommon(aName, 0, true, aCx, getter_AddRefs(request));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  request.forget(_retval);
-  return NS_OK;
+  return IDBFactoryBinding::Wrap(aCx, aScope, this);
 }
 
-NS_IMETHODIMP
-IDBFactory::Cmp(const jsval& aFirst,
-                const jsval& aSecond,
-                JSContext* aCx,
-                int16_t* _retval)
+int16_t
+IDBFactory::Cmp(JSContext* aCx, JS::Handle<JS::Value> aFirst,
+                JS::Handle<JS::Value> aSecond, ErrorResult& aRv)
 {
   Key first, second;
   nsresult rv = first.SetFromJSVal(aCx, aFirst);
   if (NS_FAILED(rv)) {
-    return rv;
+    aRv.Throw(rv);
+    return 0;
   }
 
   rv = second.SetFromJSVal(aCx, aSecond);
   if (NS_FAILED(rv)) {
-    return rv;
+    aRv.Throw(rv);
+    return 0;
   }
 
   if (first.IsUnset() || second.IsUnset()) {
-    return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+    return 0;
   }
 
-  *_retval = Key::CompareKeys(first, second);
-  return NS_OK;
+  return Key::CompareKeys(first, second);
+}
+
+already_AddRefed<nsIIDBOpenDBRequest>
+IDBFactory::OpenForPrincipal(nsIPrincipal* aPrincipal,
+                             const NonNull<nsAString>& aName,
+                             const Optional<uint64_t>& aVersion,
+                             ErrorResult& aRv)
+{
+  // Just to be on the extra-safe side
+  if (!nsContentUtils::IsCallerChrome()) {
+    MOZ_CRASH();
+  }
+
+  return Open(aPrincipal, aName, aVersion, false, aRv);
+}
+
+already_AddRefed<nsIIDBOpenDBRequest>
+IDBFactory::DeleteForPrincipal(nsIPrincipal* aPrincipal,
+                               const NonNull<nsAString>& aName,
+                               ErrorResult& aRv)
+{
+  // Just to be on the extra-safe side
+  if (!nsContentUtils::IsCallerChrome()) {
+    MOZ_CRASH();
+  }
+
+  return Open(aPrincipal, aName, Optional<uint64_t>(), true, aRv);
+}
+
+already_AddRefed<nsIIDBOpenDBRequest>
+IDBFactory::Open(nsIPrincipal* aPrincipal,
+                 const nsAString& aName, const Optional<uint64_t>& aVersion,
+                 bool aDelete, ErrorResult& aRv)
+{
+  nsresult rv;
+
+  nsCString origin;
+  if (aPrincipal) {
+    rv = QuotaManager::GetASCIIOriginFromPrincipal(aPrincipal, origin);
+    if (NS_FAILED(rv)) {
+      aRv.Throw(rv);
+      return nullptr;
+    }
+  }
+  else {
+    origin = mASCIIOrigin;
+  }
+
+  uint64_t version;
+  if (!aDelete && aVersion.WasPassed()) {
+    version = aVersion.Value();
+    if (version < 1) {
+      aRv.ThrowTypeError(MSG_INVALID_VERSION);
+      return nullptr;
+    }
+  }
+  else {
+    version = 0;
+  }
+
+  nsRefPtr<IDBOpenDBRequest> request;
+  rv = OpenInternal(aName, version, origin, aDelete,
+                    getter_AddRefs(request));
+  if (NS_FAILED(rv)) {
+    aRv.Throw(rv);
+    return nullptr;
+  }
+
+  return request.forget();
 }
