@@ -7,6 +7,8 @@
 #ifndef COMPILER_PREPROCESSOR_PREPROCESSOR_H_
 #define COMPILER_PREPROCESSOR_PREPROCESSOR_H_
 
+#include <stddef.h>
+
 #include "pp_utils.h"
 
 namespace pp
@@ -32,7 +34,7 @@ class Preprocessor
     // Each element in the length array may contain the length of the
     // corresponding string or a value less than 0 to indicate that the string
     // is null terminated.
-    bool init(int count, const char* const string[], const int length[]);
+    bool init(size_t count, const char* const string[], const int length[]);
     // Adds a pre-defined macro.
     void predefineMacro(const char* name, int value);
 
