@@ -5,24 +5,27 @@
 
 #include "VectorImage.h"
 
-#include "imgDecoderObserver.h"
-#include "SVGDocumentWrapper.h"
+#include <algorithm>
+
 #include "gfxContext.h"
+#include "gfxDrawable.h"
 #include "gfxPlatform.h"
-#include "nsPresContext.h"
-#include "nsRect.h"
+#include "gfxUtils.h"
+#include "imgDecoderObserver.h"
+#include "mozilla/AutoRestore.h"
+#include "mozilla/dom/SVGSVGElement.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIObserverService.h"
 #include "nsIPresShell.h"
 #include "nsIStreamListener.h"
 #include "nsMimeTypes.h"
-#include "nsComponentManagerUtils.h"
+#include "nsPresContext.h"
+#include "nsRect.h"
 #include "nsServiceManagerUtils.h"
-#include "nsSVGUtils.h"  // for nsSVGUtils::ConvertToSurfaceSize
+#include "nsStubDocumentObserver.h"
 #include "nsSVGEffects.h" // for nsSVGRenderingObserver
-#include "gfxDrawable.h"
-#include "gfxUtils.h"
-#include "mozilla/dom/SVGSVGElement.h"
-#include <algorithm>
+#include "nsSVGUtils.h"  // for nsSVGUtils::ConvertToSurfaceSize
+#include "SVGDocumentWrapper.h"
 
 namespace mozilla {
 
@@ -32,27 +35,24 @@ using namespace layers;
 namespace image {
 
 // Helper-class: SVGRootRenderingObserver
-class SVGRootRenderingObserver : public nsSVGRenderingObserver {
+class SVGRootRenderingObserver MOZ_FINAL : public nsSVGRenderingObserver {
 public:
   SVGRootRenderingObserver(SVGDocumentWrapper* aDocWrapper,
                            VectorImage*        aVectorImage)
-    : nsSVGRenderingObserver(),
-      mDocWrapper(aDocWrapper),
-      mVectorImage(aVectorImage)
+    : nsSVGRenderingObserver()
+    , mDocWrapper(aDocWrapper)
+    , mVectorImage(aVectorImage)
+    , mHonoringInvalidations(true)
   {
+    MOZ_ASSERT(mDocWrapper, "Need a non-null SVG document wrapper");
+    MOZ_ASSERT(mVectorImage, "Need a non-null VectorImage");
+
     StartListening();
     Element* elem = GetTarget();
-    if (elem) {
-      nsSVGEffects::AddRenderingObserver(elem, this);
-      mInObserverList = true;
-    }
-#ifdef DEBUG
-    else {
-      NS_ABORT_IF_FALSE(!mInObserverList,
-                        "Have no target, so we can't be in "
-                        "target's observer list...");
-    }
-#endif
+    MOZ_ASSERT(elem, "no root SVG node for us to observe");
+
+    nsSVGEffects::AddRenderingObserver(elem, this);
+    mInObserverList = true;
   }
 
   virtual ~SVGRootRenderingObserver()
@@ -60,25 +60,32 @@ public:
     StopListening();
   }
 
+  void ResumeHonoringInvalidations()
+  {
+    mHonoringInvalidations = true;
+  }
+
 protected:
-  virtual Element* GetTarget()
+  virtual Element* GetTarget() MOZ_OVERRIDE
   {
     return mDocWrapper->GetRootSVGElem();
   }
 
-  virtual void DoUpdate()
+  virtual void DoUpdate() MOZ_OVERRIDE
   {
     Element* elem = GetTarget();
-    if (!elem)
-      return;
+    MOZ_ASSERT(elem, "missing root SVG node");
 
-    if (!mDocWrapper->ShouldIgnoreInvalidation()) {
+    if (mHonoringInvalidations && !mDocWrapper->ShouldIgnoreInvalidation()) {
       nsIFrame* frame = elem->GetPrimaryFrame();
       if (!frame || frame->PresContext()->PresShell()->IsDestroying()) {
         // We're being destroyed. Bail out.
         return;
       }
 
+      // Ignore further invalidations until we draw.
+      mHonoringInvalidations = false;
+
       mVectorImage->InvalidateObserver();
     }
 
@@ -87,22 +94,146 @@ protected:
     if (!mInObserverList) {
       nsSVGEffects::AddRenderingObserver(elem, this);
       mInObserverList = true;
-    }
+    } 
   }
 
   // Private data
-  nsRefPtr<SVGDocumentWrapper> mDocWrapper;
-  VectorImage* mVectorImage;   // Raw pointer because it owns me.
+  const nsRefPtr<SVGDocumentWrapper> mDocWrapper;
+  VectorImage* const mVectorImage;   // Raw pointer because it owns me.
+  bool mHonoringInvalidations;
 };
 
+class SVGParseCompleteListener MOZ_FINAL : public nsStubDocumentObserver {
+public:
+  NS_DECL_ISUPPORTS
+
+  SVGParseCompleteListener(nsIDocument* aDocument,
+                           VectorImage* aImage)
+    : mDocument(aDocument)
+    , mImage(aImage)
+  {
+    MOZ_ASSERT(mDocument, "Need an SVG document");
+    MOZ_ASSERT(mImage, "Need an image");
+
+    mDocument->AddObserver(this);
+  }
+
+  ~SVGParseCompleteListener()
+  {
+    if (mDocument) {
+      // The document must have been destroyed before we got our event.
+      // Otherwise this can't happen, since documents hold strong references to
+      // their observers.
+      Cancel();
+    }
+  }
+
+  NS_IMETHOD_(void) EndLoad(nsIDocument* aDocument) MOZ_OVERRIDE
+  {
+    MOZ_ASSERT(aDocument == mDocument, "Got EndLoad for wrong document?");
+
+    // OnSVGDocumentParsed will release our owner's reference to us, so ensure
+    // we stick around long enough to complete our work.
+    nsRefPtr<SVGParseCompleteListener> kungFuDeathGroup(this);
+
+    mImage->OnSVGDocumentParsed();
+  }
+
+  void Cancel()
+  {
+    MOZ_ASSERT(mDocument, "Duplicate call to Cancel");
+    if (mDocument) {
+      mDocument->RemoveObserver(this);
+      mDocument = nullptr;
+    }
+  }
+
+private:
+  nsCOMPtr<nsIDocument> mDocument;
+  VectorImage* const mImage; // Raw pointer to owner.
+};
+
+NS_IMPL_ISUPPORTS1(SVGParseCompleteListener, nsIDocumentObserver)
+
+class SVGLoadEventListener MOZ_FINAL : public nsIDOMEventListener {
+public:
+  NS_DECL_ISUPPORTS
+
+  SVGLoadEventListener(nsIDocument* aDocument,
+                       VectorImage* aImage)
+    : mDocument(aDocument)
+    , mImage(aImage)
+  {
+    MOZ_ASSERT(mDocument, "Need an SVG document");
+    MOZ_ASSERT(mImage, "Need an image");
+
+    mDocument->AddEventListener(NS_LITERAL_STRING("MozSVGAsImageDocumentLoad"), this, true, false);
+    mDocument->AddEventListener(NS_LITERAL_STRING("SVGAbort"), this, true, false);
+    mDocument->AddEventListener(NS_LITERAL_STRING("SVGError"), this, true, false);
+  }
+
+  ~SVGLoadEventListener()
+  {
+    if (mDocument) {
+      // The document must have been destroyed before we got our event.
+      // Otherwise this can't happen, since documents hold strong references to
+      // their observers.
+      Cancel();
+    }
+  }
+
+  NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent) MOZ_OVERRIDE
+  {
+    MOZ_ASSERT(mDocument, "Need an SVG document. Received multiple events?");
+
+    // OnSVGDocumentLoaded/OnSVGDocumentError will release our owner's reference
+    // to us, so ensure we stick around long enough to complete our work.
+    nsRefPtr<SVGLoadEventListener> kungFuDeathGroup(this);
+
+    nsAutoString eventType;
+    aEvent->GetType(eventType);
+    MOZ_ASSERT(eventType.EqualsLiteral("MozSVGAsImageDocumentLoad")  ||
+               eventType.EqualsLiteral("SVGAbort")                   ||
+               eventType.EqualsLiteral("SVGError"),
+               "Received unexpected event");
+
+    if (eventType.EqualsLiteral("MozSVGAsImageDocumentLoad")) {
+      mImage->OnSVGDocumentLoaded();
+    } else {
+      mImage->OnSVGDocumentError();
+    }
+
+    return NS_OK;
+  }
+
+  void Cancel()
+  {
+    MOZ_ASSERT(mDocument, "Duplicate call to Cancel");
+    if (mDocument) {
+      mDocument->RemoveEventListener(NS_LITERAL_STRING("MozSVGAsImageDocumentLoad"), this, true);
+      mDocument->RemoveEventListener(NS_LITERAL_STRING("SVGAbort"), this, true);
+      mDocument->RemoveEventListener(NS_LITERAL_STRING("SVGError"), this, true);
+      mDocument = nullptr;
+    }
+  }
+
+private:
+  nsCOMPtr<nsIDocument> mDocument;
+  VectorImage* const mImage; // Raw pointer to owner.
+};
+
+NS_IMPL_ISUPPORTS1(SVGLoadEventListener, nsIDOMEventListener)
+
 // Helper-class: SVGDrawingCallback
 class SVGDrawingCallback : public gfxDrawingCallback {
 public:
   SVGDrawingCallback(SVGDocumentWrapper* aSVGDocumentWrapper,
                      const nsIntRect& aViewport,
+                     const gfxSize& aScale,
                      uint32_t aImageFlags) :
     mSVGDocumentWrapper(aSVGDocumentWrapper),
     mViewport(aViewport),
+    mScale(aScale),
     mImageFlags(aImageFlags)
   {}
   virtual bool operator()(gfxContext* aContext,
@@ -112,6 +243,7 @@ public:
 private:
   nsRefPtr<SVGDocumentWrapper> mSVGDocumentWrapper;
   const nsIntRect mViewport;
+  const gfxSize   mScale;
   uint32_t        mImageFlags;
 };
 
@@ -122,7 +254,7 @@ SVGDrawingCallback::operator()(gfxContext* aContext,
                                const gfxPattern::GraphicsFilter& aFilter,
                                const gfxMatrix& aTransform)
 {
-  NS_ABORT_IF_FALSE(mSVGDocumentWrapper, "need an SVGDocumentWrapper");
+  MOZ_ASSERT(mSVGDocumentWrapper, "need an SVGDocumentWrapper");
 
   // Get (& sanity-check) the helper-doc's presShell
   nsCOMPtr<nsIPresShell> presShell;
@@ -130,7 +262,7 @@ SVGDrawingCallback::operator()(gfxContext* aContext,
     NS_WARNING("Unable to draw -- presShell lookup failed");
     return false;
   }
-  NS_ABORT_IF_FALSE(presShell, "GetPresShell succeeded but returned null");
+  MOZ_ASSERT(presShell, "GetPresShell succeeded but returned null");
 
   gfxContextAutoSaveRestore contextRestorer(aContext);
 
@@ -141,9 +273,10 @@ SVGDrawingCallback::operator()(gfxContext* aContext,
 
   gfxContextMatrixAutoSaveRestore contextMatrixRestorer(aContext);
   aContext->Multiply(gfxMatrix(aTransform).Invert());
+  aContext->Scale(1.0 / mScale.width, 1.0 / mScale.height);
 
   nsPresContext* presContext = presShell->GetPresContext();
-  NS_ABORT_IF_FALSE(presContext, "pres shell w/out pres context");
+  MOZ_ASSERT(presContext, "pres shell w/out pres context");
 
   nsRect svgRect(presContext->DevPixelsToAppUnits(mViewport.x),
                  presContext->DevPixelsToAppUnits(mViewport.y),
@@ -174,17 +307,16 @@ NS_IMPL_ISUPPORTS3(VectorImage,
 VectorImage::VectorImage(imgStatusTracker* aStatusTracker,
                          nsIURI* aURI /* = nullptr */) :
   ImageResource(aStatusTracker, aURI), // invoke superclass's constructor
-  mRestrictedRegion(0, 0, 0, 0),
   mIsInitialized(false),
   mIsFullyLoaded(false),
   mIsDrawing(false),
-  mHaveAnimations(false),
-  mHaveRestrictedRegion(false)
+  mHaveAnimations(false)
 {
 }
 
 VectorImage::~VectorImage()
 {
+  CancelAllListeners();
 }
 
 //------------------------------------------------------------------------------
@@ -198,10 +330,9 @@ VectorImage::Init(const char* aMimeType,
   if (mIsInitialized)
     return NS_ERROR_ILLEGAL_VALUE;
 
-  NS_ABORT_IF_FALSE(!mIsFullyLoaded && !mHaveAnimations &&
-                    !mHaveRestrictedRegion && !mError,
-                    "Flags unexpectedly set before initialization");
-  NS_ABORT_IF_FALSE(!strcmp(aMimeType, IMAGE_SVG_XML), "Unexpected mimetype");
+  MOZ_ASSERT(!mIsFullyLoaded && !mHaveAnimations && !mError,
+             "Flags unexpectedly set before initialization");
+  MOZ_ASSERT(!strcmp(aMimeType, IMAGE_SVG_XML), "Unexpected mimetype");
 
   mIsInitialized = true;
   return NS_OK;
@@ -244,9 +375,26 @@ VectorImage::OutOfProcessSizeOfDecoded() const
 nsresult
 VectorImage::OnImageDataComplete(nsIRequest* aRequest,
                                  nsISupports* aContext,
-                                 nsresult aStatus)
+                                 nsresult aStatus,
+                                 bool aLastPart)
 {
-  return OnStopRequest(aRequest, aContext, aStatus);
+  // Call our internal OnStopRequest method, which only talks to our embedded
+  // SVG document. This won't have any effect on our imgStatusTracker.
+  nsresult finalStatus = OnStopRequest(aRequest, aContext, aStatus);
+
+  // Give precedence to Necko failure codes.
+  if (NS_FAILED(aStatus))
+    finalStatus = aStatus;
+
+  // Actually fire OnStopRequest.
+  if (mStatusTracker) {
+    nsRefPtr<imgStatusTracker> clone = mStatusTracker->CloneForRecording();
+    imgDecoderObserver* observer = clone->GetDecoderObserver();
+    observer->OnStopRequest(aLastPart, finalStatus);
+    imgStatusTracker::StatusDiff diff = mStatusTracker->CalculateAndApplyDifference(clone);
+    mStatusTracker->SyncNotifyDifference(diff);
+  }
+  return finalStatus;
 }
 
 nsresult
@@ -271,7 +419,7 @@ VectorImage::StartAnimation()
   if (mError)
     return NS_ERROR_FAILURE;
 
-  NS_ABORT_IF_FALSE(ShouldAnimate(), "Should not animate!");
+  MOZ_ASSERT(ShouldAnimate(), "Should not animate!");
 
   mSVGDocumentWrapper->StartAnimation();
   return NS_OK;
@@ -283,8 +431,8 @@ VectorImage::StopAnimation()
   if (mError)
     return NS_ERROR_FAILURE;
 
-  NS_ABORT_IF_FALSE(mIsFullyLoaded && mHaveAnimations,
-                    "Should not have been animating!");
+  MOZ_ASSERT(mIsFullyLoaded && mHaveAnimations,
+             "Should not have been animating!");
 
   mSVGDocumentWrapper->StopAnimation();
   return NS_OK;
@@ -296,6 +444,12 @@ VectorImage::ShouldAnimate()
   return ImageResource::ShouldAnimate() && mIsFullyLoaded && mHaveAnimations;
 }
 
+NS_IMETHODIMP_(void)
+VectorImage::SetAnimationStartTime(const mozilla::TimeStamp& aTime)
+{
+  // We don't care about animation start time.
+}
+
 //------------------------------------------------------------------------------
 // imgIContainer methods
 
@@ -324,6 +478,7 @@ NS_IMETHODIMP_(void)
 VectorImage::RequestRefresh(const mozilla::TimeStamp& aTime)
 {
   // TODO: Implement for b666446.
+  EvaluateAnimation();
 }
 
 //******************************************************************************
@@ -345,6 +500,44 @@ VectorImage::GetHeight(int32_t* aHeight)
   return NS_OK;
 }
 
+//******************************************************************************
+/* [noscript] readonly attribute nsSize intrinsicSize; */
+NS_IMETHODIMP
+VectorImage::GetIntrinsicSize(nsSize* aSize)
+{
+  if (mError || !mIsFullyLoaded)
+    return NS_ERROR_FAILURE;
+
+  nsIFrame* rootFrame = mSVGDocumentWrapper->GetRootLayoutFrame();
+  if (!rootFrame)
+    return NS_ERROR_FAILURE;
+
+  *aSize = nsSize(-1, -1);
+  nsIFrame::IntrinsicSize rfSize = rootFrame->GetIntrinsicSize();
+  if (rfSize.width.GetUnit() == eStyleUnit_Coord)
+    aSize->width = rfSize.width.GetCoordValue();
+  if (rfSize.height.GetUnit() == eStyleUnit_Coord)
+    aSize->height = rfSize.height.GetCoordValue();
+
+  return NS_OK;
+}
+
+//******************************************************************************
+/* [noscript] readonly attribute nsSize intrinsicRatio; */
+NS_IMETHODIMP
+VectorImage::GetIntrinsicRatio(nsSize* aRatio)
+{
+  if (mError || !mIsFullyLoaded)
+    return NS_ERROR_FAILURE;
+
+  nsIFrame* rootFrame = mSVGDocumentWrapper->GetRootLayoutFrame();
+  if (!rootFrame)
+    return NS_ERROR_FAILURE;
+
+  *aRatio = rootFrame->GetIntrinsicRatio();
+  return NS_OK;
+}
+
 //******************************************************************************
 /* readonly attribute unsigned short type; */
 NS_IMETHODIMP
@@ -376,6 +569,23 @@ VectorImage::GetAnimated(bool* aAnimated)
   return NS_OK;
 }
 
+//******************************************************************************
+/* [notxpcom] int32_t getFirstFrameDelay (); */
+int32_t
+VectorImage::GetFirstFrameDelay()
+{
+  if (mError)
+    return -1;
+
+  if (!mSVGDocumentWrapper->IsAnimated())
+    return -1;
+
+  // We don't really have a frame delay, so just pretend that we constantly
+  // need updates.
+  return 0;
+}
+
+
 //******************************************************************************
 /* [notxpcom] boolean frameIsOpaque(in uint32_t aWhichFrame); */
 NS_IMETHODIMP_(bool)
@@ -396,36 +606,7 @@ VectorImage::GetFrame(uint32_t aWhichFrame,
                       gfxASurface** _retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
-  nsRefPtr<gfxImageSurface> surface;
-  nsresult rv = CopyFrame(aWhichFrame, aFlags, getter_AddRefs(surface));
-  if (NS_SUCCEEDED(rv)) {
-    *_retval = surface.forget().get();
-  }
-  return rv;
-}
 
-//******************************************************************************
-/* [noscript] ImageContainer getImageContainer(); */
-NS_IMETHODIMP
-VectorImage::GetImageContainer(LayerManager* aManager,
-                               mozilla::layers::ImageContainer** _retval)
-{
-  *_retval = nullptr;
-  return NS_OK;
-}
-
-//******************************************************************************
-/* [noscript] gfxImageSurface copyFrame(in uint32_t aWhichFrame,
- *                                      in uint32_t aFlags); */
-NS_IMETHODIMP
-VectorImage::CopyFrame(uint32_t aWhichFrame,
-                       uint32_t aFlags,
-                       gfxImageSurface** _retval)
-{
-  NS_ENSURE_ARG_POINTER(_retval);
-  // XXXdholbert NOTE: Currently assuming FRAME_CURRENT for simplicity.
-  // Could handle FRAME_FIRST by saving helper-doc current time, seeking
-  // to time 0, rendering, and then seeking to saved time.
   if (aWhichFrame > FRAME_MAX_VALUE)
     return NS_ERROR_INVALID_ARG;
 
@@ -447,14 +628,7 @@ VectorImage::CopyFrame(uint32_t aWhichFrame,
   // ---------------------------------------------
   // Make our surface the size of what will ultimately be drawn to it.
   // (either the full image size, or the restricted region)
-  gfxIntSize surfaceSize;
-  if (mHaveRestrictedRegion) {
-    surfaceSize.width = mRestrictedRegion.width;
-    surfaceSize.height = mRestrictedRegion.height;
-  } else {
-    surfaceSize.width = imageIntSize.width;
-    surfaceSize.height = imageIntSize.height;
-  }
+  gfxIntSize surfaceSize(imageIntSize.width, imageIntSize.height);
 
   nsRefPtr<gfxImageSurface> surface =
     new gfxImageSurface(surfaceSize, gfxASurface::ImageFormatARGB32);
@@ -466,63 +640,23 @@ VectorImage::CopyFrame(uint32_t aWhichFrame,
                      gfxRect(gfxPoint(0,0), gfxIntSize(imageIntSize.width,
                                                        imageIntSize.height)),
                      nsIntRect(nsIntPoint(0,0), imageIntSize),
-                     imageIntSize, aFlags);
-  if (NS_SUCCEEDED(rv)) {
-    *_retval = surface.forget().get();
-  }
+                     imageIntSize, nullptr, aWhichFrame, aFlags);
 
+  NS_ENSURE_SUCCESS(rv, rv);
+  *_retval = surface.forget().get();
   return rv;
 }
 
 //******************************************************************************
-/* [noscript] imgIContainer extractFrame(uint32_t aWhichFrame,
- *                                       [const] in nsIntRect aRegion,
- *                                       in uint32_t aFlags); */
+/* [noscript] ImageContainer getImageContainer(); */
 NS_IMETHODIMP
-VectorImage::ExtractFrame(uint32_t aWhichFrame,
-                          const nsIntRect& aRegion,
-                          uint32_t aFlags,
-                          imgIContainer** _retval)
+VectorImage::GetImageContainer(LayerManager* aManager,
+                               mozilla::layers::ImageContainer** _retval)
 {
-  NS_ENSURE_ARG_POINTER(_retval);
-  if (mError || !mIsFullyLoaded)
-    return NS_ERROR_FAILURE;
-
-  // XXXdholbert NOTE: This method assumes FRAME_CURRENT (not FRAME_FIRST)
-  // right now, because mozilla doesn't actually contain any clients of this
-  // method that use FRAME_FIRST.  If it's needed, we *could* handle
-  // FRAME_FIRST by saving the helper-doc's current SMIL time, seeking it to
-  // time 0, rendering to a RasterImage, and then restoring our saved time.
-  if (aWhichFrame != FRAME_CURRENT) {
-    NS_WARNING("VectorImage::ExtractFrame with something other than "
-               "FRAME_CURRENT isn't supported yet. Assuming FRAME_CURRENT.");
-  }
-
-  // XXXdholbert This method also doesn't actually freeze animation in the
-  // returned imgIContainer, because it shares our helper-document. To
-  // get a true snapshot, we need to clone the document - see bug 590792.
-
-  // Make a new container with same SVG document.
-  nsRefPtr<VectorImage> extractedImg = new VectorImage();
-  extractedImg->mSVGDocumentWrapper = mSVGDocumentWrapper;
-  extractedImg->mAnimationMode = kDontAnimMode;
-
-  extractedImg->mRestrictedRegion.x = aRegion.x;
-  extractedImg->mRestrictedRegion.y = aRegion.y;
-
-  // (disallow negative width/height on our restricted region)
-  extractedImg->mRestrictedRegion.width  = std::max(aRegion.width,  0);
-  extractedImg->mRestrictedRegion.height = std::max(aRegion.height, 0);
-
-  extractedImg->mIsInitialized = true;
-  extractedImg->mIsFullyLoaded = true;
-  extractedImg->mHaveRestrictedRegion = true;
-
-  *_retval = extractedImg.forget().get();
+  *_retval = nullptr;
   return NS_OK;
 }
 
-
 //******************************************************************************
 /* [noscript] void draw(in gfxContext aContext,
  *                      in gfxGraphicsFilter aFilter,
@@ -530,6 +664,8 @@ VectorImage::ExtractFrame(uint32_t aWhichFrame,
  *                      [const] in gfxRect aFill,
  *                      [const] in nsIntRect aSubimage,
  *                      [const] in nsIntSize aViewportSize,
+ *                      [const] in SVGImageContext aSVGContext,
+ *                      in uint32_t aWhichFrame,
  *                      in uint32_t aFlags); */
 NS_IMETHODIMP
 VectorImage::Draw(gfxContext* aContext,
@@ -538,8 +674,13 @@ VectorImage::Draw(gfxContext* aContext,
                   const gfxRect& aFill,
                   const nsIntRect& aSubimage,
                   const nsIntSize& aViewportSize,
+                  const SVGImageContext* aSVGContext,
+                  uint32_t aWhichFrame,
                   uint32_t aFlags)
 {
+  if (aWhichFrame > FRAME_MAX_VALUE)
+    return NS_ERROR_INVALID_ARG;
+
   NS_ENSURE_ARG_POINTER(aContext);
   if (mError || !mIsFullyLoaded)
     return NS_ERROR_FAILURE;
@@ -548,54 +689,68 @@ VectorImage::Draw(gfxContext* aContext,
     NS_WARNING("Refusing to make re-entrant call to VectorImage::Draw");
     return NS_ERROR_FAILURE;
   }
+
+  if (mAnimationConsumers == 0 && mStatusTracker) {
+    mStatusTracker->OnUnlockedDraw();
+  }
+
+  AutoRestore<bool> autoRestoreIsDrawing(mIsDrawing);
   mIsDrawing = true;
 
+  float time = aWhichFrame == FRAME_FIRST ? 0.0f
+                                          : mSVGDocumentWrapper->GetCurrentTime();
+  AutoSVGRenderingState autoSVGState(aSVGContext,
+                                     time,
+                                     mSVGDocumentWrapper->GetRootSVGElem());
+
+  // gfxUtils::DrawPixelSnapped may rasterize this image to a temporary surface
+  // if we hit the tiling path. Unfortunately, the temporary surface isn't
+  // created at the size at which we'll ultimately draw, causing fuzzy output.
+  // To fix this we pre-apply the transform's scaling to the drawing parameters
+  // and remove the scaling from the transform, so the fact that temporary
+  // surfaces won't take the scaling into account doesn't matter. (Bug 600207.)
+  gfxSize scale(aUserSpaceToImageSpace.ScaleFactors(true));
+  gfxPoint translation(aUserSpaceToImageSpace.GetTranslation());
+
+  // Remove the scaling from the transform.
+  gfxMatrix unscale;
+  unscale.Translate(gfxPoint(translation.x / scale.width,
+                             translation.y / scale.height));
+  unscale.Scale(1.0 / scale.width, 1.0 / scale.height);
+  unscale.Translate(-translation);
+  gfxMatrix unscaledTransform(aUserSpaceToImageSpace * unscale);
+
   mSVGDocumentWrapper->UpdateViewportBounds(aViewportSize);
   mSVGDocumentWrapper->FlushImageTransformInvalidation();
 
-  nsIntSize imageSize = mHaveRestrictedRegion ?
-    mRestrictedRegion.Size() : aViewportSize;
-
-  // XXXdholbert Do we need to convert image size from
-  // CSS pixels to dev pixels here? (is gfxCallbackDrawable's 2nd arg in dev
-  // pixels?)
-  gfxIntSize imageSizeGfx(imageSize.width, imageSize.height);
-
-  // Based on imgFrame::Draw
-  gfxRect sourceRect = aUserSpaceToImageSpace.Transform(aFill);
-  gfxRect imageRect(0, 0, imageSize.width, imageSize.height);
-  gfxRect subimage(aSubimage.x, aSubimage.y, aSubimage.width, aSubimage.height);
-
+  // Rescale drawing parameters.
+  gfxIntSize drawableSize(aViewportSize.width / scale.width,
+                          aViewportSize.height / scale.height);
+  gfxRect drawableSourceRect = unscaledTransform.Transform(aFill);
+  gfxRect drawableImageRect(0, 0, drawableSize.width, drawableSize.height);
+  gfxRect drawableSubimage(aSubimage.x, aSubimage.y,
+                           aSubimage.width, aSubimage.height);
+  drawableSubimage.ScaleRoundOut(1.0 / scale.width, 1.0 / scale.height);
 
   nsRefPtr<gfxDrawingCallback> cb =
     new SVGDrawingCallback(mSVGDocumentWrapper,
-                           mHaveRestrictedRegion ?
-                           mRestrictedRegion :
                            nsIntRect(nsIntPoint(0, 0), aViewportSize),
+                           scale,
                            aFlags);
 
-  nsRefPtr<gfxDrawable> drawable = new gfxCallbackDrawable(cb, imageSizeGfx);
+  nsRefPtr<gfxDrawable> drawable = new gfxCallbackDrawable(cb, drawableSize);
 
-  gfxUtils::DrawPixelSnapped(aContext, drawable,
-                             aUserSpaceToImageSpace,
-                             subimage, sourceRect, imageRect, aFill,
-                             gfxASurface::ImageFormatARGB32, aFilter);
+  gfxUtils::DrawPixelSnapped(aContext, drawable, unscaledTransform,
+                             drawableSubimage, drawableSourceRect,
+                             drawableImageRect, aFill,
+                             gfxASurface::ImageFormatARGB32, aFilter, aFlags);
+
+  MOZ_ASSERT(mRenderingObserver, "Should have a rendering observer by now");
+  mRenderingObserver->ResumeHonoringInvalidations();
 
-  mIsDrawing = false;
   return NS_OK;
 }
 
-//******************************************************************************
-/* [notxpcom] nsIFrame GetRootLayoutFrame() */
-nsIFrame*
-VectorImage::GetRootLayoutFrame()
-{
-  if (mError)
-    return nullptr;
-
-  return mSVGDocumentWrapper->GetRootLayoutFrame();
-}
-
 //******************************************************************************
 /* void requestDecode() */
 NS_IMETHODIMP
@@ -612,6 +767,11 @@ VectorImage::StartDecoding()
   return NS_OK;
 }
 
+bool
+VectorImage::IsDecoded()
+{
+  return mIsFullyLoaded || mError;
+}
 
 //******************************************************************************
 /* void lockImage() */
@@ -657,6 +817,15 @@ VectorImage::ResetAnimation()
   return NS_OK;
 }
 
+NS_IMETHODIMP_(float)
+VectorImage::GetFrameIndex(uint32_t aWhichFrame)
+{
+  MOZ_ASSERT(aWhichFrame <= FRAME_MAX_VALUE, "Invalid argument");
+  return aWhichFrame == FRAME_FIRST
+         ? 0.0f
+         : mSVGDocumentWrapper->GetCurrentTime();
+}
+
 //------------------------------------------------------------------------------
 // nsIRequestObserver methods
 
@@ -665,17 +834,39 @@ VectorImage::ResetAnimation()
 NS_IMETHODIMP
 VectorImage::OnStartRequest(nsIRequest* aRequest, nsISupports* aCtxt)
 {
-  NS_ABORT_IF_FALSE(!mSVGDocumentWrapper,
-                    "Repeated call to OnStartRequest -- can this happen?");
+  MOZ_ASSERT(!mSVGDocumentWrapper,
+             "Repeated call to OnStartRequest -- can this happen?");
 
   mSVGDocumentWrapper = new SVGDocumentWrapper();
   nsresult rv = mSVGDocumentWrapper->OnStartRequest(aRequest, aCtxt);
   if (NS_FAILED(rv)) {
     mSVGDocumentWrapper = nullptr;
     mError = true;
+    return rv;
   }
 
-  return rv;
+  // Sending StartDecode will block page load until the document's ready.  (We
+  // unblock it by sending StopDecode in OnSVGDocumentLoaded or
+  // OnSVGDocumentError.)
+  if (mStatusTracker) {
+    nsRefPtr<imgStatusTracker> clone = mStatusTracker->CloneForRecording();
+    imgDecoderObserver* observer = clone->GetDecoderObserver();
+    observer->OnStartDecode();
+    imgStatusTracker::StatusDiff diff = mStatusTracker->CalculateAndApplyDifference(clone);
+    mStatusTracker->SyncNotifyDifference(diff);
+  }
+
+  // Create a listener to wait until the SVG document is fully loaded, which
+  // will signal that this image is ready to render. Certain error conditions
+  // will prevent us from ever getting this notification, so we also create a
+  // listener that waits for parsing to complete and cancels the
+  // SVGLoadEventListener if needed. The listeners are automatically attached
+  // to the document by their constructors.
+  nsIDocument* document = mSVGDocumentWrapper->GetDocument();
+  mLoadEventListener = new SVGLoadEventListener(document, this);
+  mParseCompleteListener = new SVGParseCompleteListener(document, this);
+
+  return NS_OK;
 }
 
 //******************************************************************************
@@ -688,38 +879,93 @@ VectorImage::OnStopRequest(nsIRequest* aRequest, nsISupports* aCtxt,
   if (mError)
     return NS_ERROR_FAILURE;
 
-  NS_ABORT_IF_FALSE(!mIsFullyLoaded && !mHaveAnimations,
-                    "these flags shouldn't get set until OnStopRequest. "
-                    "Duplicate calls to OnStopRequest?");
+  return mSVGDocumentWrapper->OnStopRequest(aRequest, aCtxt, aStatus);
+}
 
-  nsresult rv = mSVGDocumentWrapper->OnStopRequest(aRequest, aCtxt, aStatus);
-  if (!mSVGDocumentWrapper->ParsedSuccessfully()) {
-    // XXXdholbert Need to do something more here -- right now, this just
-    // makes us draw the "object" icon, rather than the (jagged) "broken image"
-    // icon.  See bug 594505.
-    mError = true;
-    return rv;
+void
+VectorImage::OnSVGDocumentParsed()
+{
+  MOZ_ASSERT(mParseCompleteListener, "Should have the parse complete listener");
+  MOZ_ASSERT(mLoadEventListener, "Should have the load event listener");
+
+  if (!mSVGDocumentWrapper->GetRootSVGElem()) {
+    // This is an invalid SVG document. It may have failed to parse, or it may
+    // be missing the <svg> root element, or the <svg> root element may not
+    // declare the correct namespace. In any of these cases, we'll never be
+    // notified that the SVG finished loading, so we need to treat this as an error.
+    OnSVGDocumentError();
   }
+}
+
+void
+VectorImage::CancelAllListeners()
+{
+  if (mParseCompleteListener) {
+    mParseCompleteListener->Cancel();
+    mParseCompleteListener = nullptr;
+  }
+  if (mLoadEventListener) {
+    mLoadEventListener->Cancel();
+    mLoadEventListener = nullptr;
+  }
+}
+
+void
+VectorImage::OnSVGDocumentLoaded()
+{
+  MOZ_ASSERT(mSVGDocumentWrapper->GetRootSVGElem(),
+             "Should have parsed successfully");
+  MOZ_ASSERT(!mIsFullyLoaded && !mHaveAnimations,
+             "These flags shouldn't get set until OnSVGDocumentLoaded. "
+             "Duplicate calls to OnSVGDocumentLoaded?");
+
+  CancelAllListeners();
+
+  // XXX Flushing is wasteful if embedding frame hasn't had initial reflow.
+  mSVGDocumentWrapper->FlushLayout();
 
   mIsFullyLoaded = true;
   mHaveAnimations = mSVGDocumentWrapper->IsAnimated();
 
-  // Start listening to our image for rendering updates
+  // Start listening to our image for rendering updates.
   mRenderingObserver = new SVGRootRenderingObserver(mSVGDocumentWrapper, this);
 
-  // Tell *our* observers that we're done loading
+  // Tell *our* observers that we're done loading.
   if (mStatusTracker) {
-    // NOTE: This signals that width/height are available.
-    imgDecoderObserver* observer = mStatusTracker->GetDecoderObserver();
-    observer->OnStartContainer();
+    nsRefPtr<imgStatusTracker> clone = mStatusTracker->CloneForRecording();
+    imgDecoderObserver* observer = clone->GetDecoderObserver();
 
+    observer->OnStartContainer(); // Signal that width/height are available.
     observer->FrameChanged(&nsIntRect::GetMaxSizedIntRect());
     observer->OnStopFrame();
-    observer->OnStopDecode(NS_OK);
-  }
-  EvaluateAnimation();
+    observer->OnStopDecode(NS_OK); // Unblock page load.
 
-  return rv;
+    imgStatusTracker::StatusDiff diff = mStatusTracker->CalculateAndApplyDifference(clone);
+    mStatusTracker->SyncNotifyDifference(diff);
+  }
+
+  EvaluateAnimation();
+}
+
+void
+VectorImage::OnSVGDocumentError()
+{
+  CancelAllListeners();
+
+  // XXXdholbert Need to do something more for the parsing failed case -- right
+  // now, this just makes us draw the "object" icon, rather than the (jagged)
+  // "broken image" icon.  See bug 594505.
+  mError = true;
+
+  if (mStatusTracker) {
+    nsRefPtr<imgStatusTracker> clone = mStatusTracker->CloneForRecording();
+    imgDecoderObserver* observer = clone->GetDecoderObserver();
+
+    // Unblock page load.
+    observer->OnStopDecode(NS_ERROR_FAILURE);
+    imgStatusTracker::StatusDiff diff = mStatusTracker->CalculateAndApplyDifference(clone);
+    mStatusTracker->SyncNotifyDifference(diff);
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -748,9 +994,8 @@ void
 VectorImage::InvalidateObserver()
 {
   if (mStatusTracker) {
-    imgDecoderObserver* observer = mStatusTracker->GetDecoderObserver();
-    observer->FrameChanged(&nsIntRect::GetMaxSizedIntRect());
-    observer->OnStopFrame();
+    mStatusTracker->FrameChanged(&nsIntRect::GetMaxSizedIntRect());
+    mStatusTracker->OnStopFrame();
   }
 }
 
