@@ -1,4 +1,4 @@
-// |jit-test| mjitalways
+// |jit-test| slow; mjitalways
 
 var nlocals = 50;
 var localstr = "";
@@ -16,7 +16,9 @@ var body = localstr +
            "if (x == 0) return; " +
            "arr[3] = (new Function(arg, body));" +
            "for (var i = 0; i < 4; ++i) arr[i](x-1);";
-(new Function(arg, body))(1000);
+
+// XXX interpreter bailouts during recursion below can cause us to hit the limit quickly.
+try { (new Function(arg, body))(1000); } catch (e) {}
 
 /*
  * Also check for OOM in CompileFunction. To avoid taking 5 seconds, use a
