@@ -5,7 +5,7 @@
 
 
 /**
-   File Name:          7.5-9-n.js
+   File Name:          7.5-4-n.js
    ECMA Section:       7.5 Identifiers
    Description:        Identifiers are of unlimited length
    - can contain letters, a decimal digit, _, or $
@@ -15,16 +15,16 @@
    Author:             christine@netscape.com
    Date:               11 september 1997
 */
-var SECTION = "7.5-9-n";
+var SECTION = "7.5-4-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Identifiers";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var 123=\"hi\"";
+DESCRIPTION = "var 2abc";
 EXPECTED = "error";
 
-new TestCase( SECTION,    "var 123=\"hi\"",   "error",    eval("var 123 = \"hi\";array[item] = 123;") );
+new TestCase( SECTION,    "var 2abc",   "error",    eval("var 2abc") );
 
 test();
