@@ -4,10 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 351102;
-var summary = 'try/catch-guard/finally GC issues';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 355052;
+var summary = 'Do not crash with valueOf:gc and __iterator__';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
 //-----------------------------------------------------------------------------
@@ -19,16 +19,25 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var f;
+
+  expect = /TypeError: .+ is not a function/;
+  actual = 'No Error';
   try
   {
-    try { d.d.d } catch([] if gc()) { }
+    var obj = {valueOf: gc };
+
+    function f() {
+      ( obj * [].a )();
+    }
+
+    f();
   }
   catch(ex)
   {
+    actual = ex + '';
   }
-  reportCompare(expect, actual, summary + ': 5');
+
+  reportMatch(expect, actual, summary);
 
   exitFunc ('test');
 }
