@@ -8,17 +8,33 @@
  * object that is a containing block for them
  */
 
-#include "nsCOMPtr.h"
 #include "nsAbsoluteContainingBlock.h"
+
 #include "nsContainerFrame.h"
+#include "nsGkAtoms.h"
 #include "nsIPresShell.h"
 #include "nsHTMLParts.h"
+#include "nsHTMLReflowState.h"
 #include "nsPresContext.h"
 #include "nsFrameManager.h"
 #include "nsCSSFrameConstructor.h"
+#include "nsIDocumentInlines.h"
 
 #ifdef DEBUG
 #include "nsBlockFrame.h"
+
+static void PrettyUC(nscoord aSize, char* aBuf)
+{
+  if (NS_UNCONSTRAINEDSIZE == aSize) {
+    strcpy(aBuf, "UC");
+  } else {
+    if((int32_t)0xdeadbeef == aSize) {
+      strcpy(aBuf, "deadbeef");
+    } else {
+      sprintf(aBuf, "%d", aSize);
+    }
+  }
+}
 #endif
 
 nsresult
@@ -26,7 +42,7 @@ nsAbsoluteContainingBlock::SetInitialChildList(nsIFrame*       aDelegatingFrame,
                                                ChildListID     aListID,
                                                nsFrameList&    aChildList)
 {
-  NS_PRECONDITION(GetChildListID() == aListID, "unexpected child list name");
+  NS_PRECONDITION(mChildListID == aListID, "unexpected child list name");
 #ifdef DEBUG
   nsFrame::VerifyDirtyBitSet(aChildList);
 #endif
@@ -39,7 +55,7 @@ nsAbsoluteContainingBlock::AppendFrames(nsIFrame*      aDelegatingFrame,
                                         ChildListID    aListID,
                                         nsFrameList&   aFrameList)
 {
-  NS_ASSERTION(GetChildListID() == aListID, "unexpected child list");
+  NS_ASSERTION(mChildListID == aListID, "unexpected child list");
 
   // Append the frames to our list of absolutely positioned frames
 #ifdef DEBUG
@@ -62,7 +78,7 @@ nsAbsoluteContainingBlock::InsertFrames(nsIFrame*      aDelegatingFrame,
                                         nsIFrame*      aPrevFrame,
                                         nsFrameList&   aFrameList)
 {
-  NS_ASSERTION(GetChildListID() == aListID, "unexpected child list");
+  NS_ASSERTION(mChildListID == aListID, "unexpected child list");
   NS_ASSERTION(!aPrevFrame || aPrevFrame->GetParent() == aDelegatingFrame,
                "inserting after sibling frame with different parent");
 
@@ -85,7 +101,7 @@ nsAbsoluteContainingBlock::RemoveFrame(nsIFrame*       aDelegatingFrame,
                                        ChildListID     aListID,
                                        nsIFrame*       aOldFrame)
 {
-  NS_ASSERTION(GetChildListID() == aListID, "unexpected child list");
+  NS_ASSERTION(mChildListID == aListID, "unexpected child list");
   nsIFrame* nif = aOldFrame->GetNextInFlow();
   if (nif) {
     static_cast<nsContainerFrame*>(nif->GetParent())
@@ -100,8 +116,7 @@ nsAbsoluteContainingBlock::Reflow(nsContainerFrame*        aDelegatingFrame,
                                   nsPresContext*           aPresContext,
                                   const nsHTMLReflowState& aReflowState,
                                   nsReflowStatus&          aReflowStatus,
-                                  nscoord                  aContainingBlockWidth,
-                                  nscoord                  aContainingBlockHeight,
+                                  const nsRect&            aContainingBlock,
                                   bool                     aConstrainHeight,
                                   bool                     aCBWidthChanged,
                                   bool                     aCBHeightChanged,
@@ -120,16 +135,16 @@ nsAbsoluteContainingBlock::Reflow(nsContainerFrame*        aDelegatingFrame,
       // Reflow the frame
       nsReflowStatus  kidStatus = NS_FRAME_COMPLETE;
       ReflowAbsoluteFrame(aDelegatingFrame, aPresContext, aReflowState,
-                          aContainingBlockWidth, aContainingBlockHeight,
+                          aContainingBlock,
                           aConstrainHeight, kidFrame, kidStatus,
                           aOverflowAreas);
       nsIFrame* nextFrame = kidFrame->GetNextInFlow();
       if (!NS_FRAME_IS_FULLY_COMPLETE(kidStatus)) {
         // Need a continuation
         if (!nextFrame) {
-          nsresult rv = aPresContext->PresShell()->FrameConstructor()->
-            CreateContinuingFrame(aPresContext, kidFrame, aDelegatingFrame, &nextFrame);
-          NS_ENSURE_SUCCESS(rv, rv);
+          nextFrame =
+            aPresContext->PresShell()->FrameConstructor()->
+              CreateContinuingFrame(aPresContext, kidFrame, aDelegatingFrame);
         }
         // Add it as an overflow container.
         //XXXfr This is a hack to fix some of our printing dataloss.
@@ -141,7 +156,7 @@ nsAbsoluteContainingBlock::Reflow(nsContainerFrame*        aDelegatingFrame,
       else {
         // Delete any continuations
         if (nextFrame) {
-          tracker.Finish(kidFrame);
+          nsOverflowContinuationTracker::AutoFinish fini(&tracker, kidFrame);
           static_cast<nsContainerFrame*>(nextFrame->GetParent())
             ->DeleteNextInFlowChild(aPresContext, nextFrame, true);
         }
@@ -328,15 +343,14 @@ nsAbsoluteContainingBlock::DoMarkFramesDirty(bool aMarkAllDirty)
 // reflow...
 
 // When bug 154892 is checked in, make sure that when 
-// GetChildListID() == kFixedList, the height is unconstrained.
+// mChildListID == kFixedList, the height is unconstrained.
 // since we don't allow replicated frames to split.
 
 nsresult
 nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegatingFrame,
-                                               nsPresContext*          aPresContext,
+                                               nsPresContext*           aPresContext,
                                                const nsHTMLReflowState& aReflowState,
-                                               nscoord                  aContainingBlockWidth,
-                                               nscoord                  aContainingBlockHeight,
+                                               const nsRect&            aContainingBlock,
                                                bool                     aConstrainHeight,
                                                nsIFrame*                aKidFrame,
                                                nsReflowStatus&          aStatus,
@@ -364,11 +378,7 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
   AutoNoisyIndenter indent(nsBlockFrame::gNoisy);
 #endif // DEBUG
 
-  nsresult  rv;
-  // Get the border values
-  const nsMargin& border = aReflowState.mStyleBorder->GetComputedBorder();
-
-  nscoord availWidth = aContainingBlockWidth;
+  nscoord availWidth = aContainingBlock.width;
   if (availWidth == -1) {
     NS_ASSERTION(aReflowState.ComputedWidth() != NS_UNCONSTRAINEDSIZE,
                  "Must have a useful width _somewhere_");
@@ -379,12 +389,15 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
   nsHTMLReflowMetrics kidDesiredSize;
   nsHTMLReflowState kidReflowState(aPresContext, aReflowState, aKidFrame,
                                    nsSize(availWidth, NS_UNCONSTRAINEDSIZE),
-                                   aContainingBlockWidth,
-                                   aContainingBlockHeight);
+                                   aContainingBlock.width,
+                                   aContainingBlock.height);
 
   // Send the WillReflow() notification and position the frame
   aKidFrame->WillReflow(aPresContext);
 
+  // Get the border values
+  const nsMargin& border = aReflowState.mStyleBorder->GetComputedBorder();
+
   bool constrainHeight = (aReflowState.availableHeight != NS_UNCONSTRAINEDSIZE)
     && aConstrainHeight
        // Don't split if told not to (e.g. for fixed frames)
@@ -402,12 +415,15 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
   }
 
   // Do the reflow
-  rv = aKidFrame->Reflow(aPresContext, kidDesiredSize, kidReflowState, aStatus);
+  nsresult rv = aKidFrame->Reflow(aPresContext, kidDesiredSize, kidReflowState, aStatus);
 
   // If we're solving for 'left' or 'top', then compute it now that we know the
   // width/height
   if ((NS_AUTOOFFSET == kidReflowState.mComputedOffsets.left) ||
       (NS_AUTOOFFSET == kidReflowState.mComputedOffsets.top)) {
+    nscoord aContainingBlockWidth = aContainingBlock.width;
+    nscoord aContainingBlockHeight = aContainingBlock.height;
+
     if (-1 == aContainingBlockWidth) {
       // Get the containing block width/height
       kidReflowState.ComputeContainingBlockRectangle(aPresContext,
@@ -438,6 +454,21 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
   nsRect  rect(border.left + kidReflowState.mComputedOffsets.left + kidReflowState.mComputedMargin.left,
                border.top + kidReflowState.mComputedOffsets.top + kidReflowState.mComputedMargin.top,
                kidDesiredSize.width, kidDesiredSize.height);
+
+  // Offset the frame rect by the given origin of the absolute containing block.
+  // If the frame is auto-positioned on both sides of an axis, it will be
+  // positioned based on its containing block and we don't need to offset.
+  if (aContainingBlock.TopLeft() != nsPoint(0, 0)) {
+    if (!(kidReflowState.mStylePosition->mOffset.GetLeftUnit() == eStyleUnit_Auto &&
+          kidReflowState.mStylePosition->mOffset.GetRightUnit() == eStyleUnit_Auto)) {
+      rect.x += aContainingBlock.x;
+    }
+    if (!(kidReflowState.mStylePosition->mOffset.GetTopUnit() == eStyleUnit_Auto &&
+          kidReflowState.mStylePosition->mOffset.GetBottomUnit() == eStyleUnit_Auto)) {
+      rect.y += aContainingBlock.y;
+    }
+  }
+
   aKidFrame->SetRect(rect);
 
   nsView* view = aKidFrame->GetView();
@@ -472,22 +503,3 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
 
   return rv;
 }
-
-#ifdef DEBUG
- void nsAbsoluteContainingBlock::PrettyUC(nscoord aSize,
-                        char*   aBuf)
-{
-  if (NS_UNCONSTRAINEDSIZE == aSize) {
-    strcpy(aBuf, "UC");
-  }
-  else {
-    if((int32_t)0xdeadbeef == aSize)
-    {
-      strcpy(aBuf, "deadbeef");
-    }
-    else {
-      sprintf(aBuf, "%d", aSize);
-    }
-  }
-}
-#endif
