@@ -12,13 +12,12 @@
 
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsISupportsArray.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMNodeFilter.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMCharacterData.h"
 #include "nsIDOMAttr.h"
-#include "nsIDOMNamedNodeMap.h"
+#include "nsIDOMMozNamedAttrMap.h"
 #include "nsIDOMMutationEvent.h"
 #include "nsBindingManager.h"
 #include "nsINameSpaceManager.h"
@@ -91,25 +90,6 @@ inDOMView::~inDOMView()
   SetRootNode(nullptr);
 }
 
-#define DOMVIEW_ATOM(name_, value_) nsIAtom* inDOMView::name_ = nullptr;
-#include "inDOMViewAtomList.h"
-#undef DOMVIEW_ATOM
-
-#define DOMVIEW_ATOM(name_, value_) NS_STATIC_ATOM_BUFFER(name_##_buffer, value_)
-#include "inDOMViewAtomList.h"
-#undef DOMVIEW_ATOM
-
-/* static */ const nsStaticAtom inDOMView::Atoms_info[] = {
-#define DOMVIEW_ATOM(name_, value_) NS_STATIC_ATOM(name_##_buffer, &inDOMView::name_),
-#include "inDOMViewAtomList.h"
-#undef DOMVIEW_ATOM
-};
-
-/* static */ void
-inDOMView::InitAtoms()
-{
-  NS_RegisterStaticAtoms(Atoms_info);
-}
 
 ////////////////////////////////////////////////////////////////////////
 // nsISupports
@@ -289,13 +269,14 @@ inDOMView::GetRowCount(int32_t *aRowCount)
 }
 
 NS_IMETHODIMP
-inDOMView::GetRowProperties(int32_t index, nsISupportsArray *properties)
+inDOMView::GetRowProperties(int32_t index, nsAString& aProps)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-inDOMView::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *properties)
+inDOMView::GetCellProperties(int32_t row, nsITreeColumn* col,
+                             nsAString& aProps)
 {
   inDOMViewNode* node = nullptr;
   RowToNode(row, &node);
@@ -303,47 +284,47 @@ inDOMView::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *
 
   nsCOMPtr<nsIContent> content = do_QueryInterface(node->node);
   if (content && content->IsInAnonymousSubtree()) {
-    properties->AppendElement(kAnonymousAtom);
+    aProps.AppendLiteral("anonymous ");
   }
 
   uint16_t nodeType;
   node->node->GetNodeType(&nodeType);
   switch (nodeType) {
     case nsIDOMNode::ELEMENT_NODE:
-      properties->AppendElement(kElementNodeAtom);
+      aProps.AppendLiteral("ELEMENT_NODE");
       break;
     case nsIDOMNode::ATTRIBUTE_NODE:
-      properties->AppendElement(kAttributeNodeAtom);
+      aProps.AppendLiteral("ATTRIBUTE_NODE");
       break;
     case nsIDOMNode::TEXT_NODE:
-      properties->AppendElement(kTextNodeAtom);
+      aProps.AppendLiteral("TEXT_NODE");
       break;
     case nsIDOMNode::CDATA_SECTION_NODE:
-      properties->AppendElement(kCDataSectionNodeAtom);
+      aProps.AppendLiteral("CDATA_SECTION_NODE");
       break;
     case nsIDOMNode::ENTITY_REFERENCE_NODE:
-      properties->AppendElement(kEntityReferenceNodeAtom);
+      aProps.AppendLiteral("ENTITY_REFERENCE_NODE");
       break;
     case nsIDOMNode::ENTITY_NODE:
-      properties->AppendElement(kEntityNodeAtom);
+      aProps.AppendLiteral("ENTITY_NODE");
       break;
     case nsIDOMNode::PROCESSING_INSTRUCTION_NODE:
-      properties->AppendElement(kProcessingInstructionNodeAtom);
+      aProps.AppendLiteral("PROCESSING_INSTRUCTION_NODE");
       break;
     case nsIDOMNode::COMMENT_NODE:
-      properties->AppendElement(kCommentNodeAtom);
+      aProps.AppendLiteral("COMMENT_NODE");
       break;
     case nsIDOMNode::DOCUMENT_NODE:
-      properties->AppendElement(kDocumentNodeAtom);
+      aProps.AppendLiteral("DOCUMENT_NODE");
       break;
     case nsIDOMNode::DOCUMENT_TYPE_NODE:
-      properties->AppendElement(kDocumentTypeNodeAtom);
+      aProps.AppendLiteral("DOCUMENT_TYPE_NODE");
       break;
     case nsIDOMNode::DOCUMENT_FRAGMENT_NODE:
-      properties->AppendElement(kDocumentFragmentNodeAtom);
+      aProps.AppendLiteral("DOCUMENT_FRAGMENT_NODE");
       break;
     case nsIDOMNode::NOTATION_NODE:
-      properties->AppendElement(kNotationNodeAtom);
+      aProps.AppendLiteral("NOTATION_NODE");
       break;
   }
 
@@ -357,7 +338,7 @@ inDOMView::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *
     nsresult rv =
       accService->GetAccessibleFor(node->node, getter_AddRefs(accessible));
     if (NS_SUCCEEDED(rv) && accessible)
-      properties->AppendElement(kAccessibleNodeAtom);
+      aProps.AppendLiteral(" ACCESSIBLE_NODE");
   }
 #endif
 
@@ -365,7 +346,7 @@ inDOMView::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *
 }
 
 NS_IMETHODIMP
-inDOMView::GetColumnProperties(nsITreeColumn* col, nsISupportsArray *properties)
+inDOMView::GetColumnProperties(nsITreeColumn* col, nsAString& aProps)
 {
   return NS_OK;
 }
@@ -668,7 +649,6 @@ inDOMView::AttributeChanged(nsIDocument* aDocument, dom::Element* aElement,
   nsCOMPtr<nsIMutationObserver> kungFuDeathGrip(this);
   
   // get the dom attribute node, if there is any
-  nsCOMPtr<nsIDOMNode> content(do_QueryInterface(aElement));
   nsCOMPtr<nsIDOMElement> el(do_QueryInterface(aElement));
   nsCOMPtr<nsIDOMAttr> domAttr;
   nsDependentAtomString attrStr(aAttribute);
@@ -702,21 +682,21 @@ inDOMView::AttributeChanged(nsIDocument* aDocument, dom::Element* aElement,
       return;
     }
     // get the number of attributes on this content node
-    nsCOMPtr<nsIDOMNamedNodeMap> attrs;
-    content->GetAttributes(getter_AddRefs(attrs));
+    nsCOMPtr<nsIDOMMozNamedAttrMap> attrs;
+    el->GetAttributes(getter_AddRefs(attrs));
     uint32_t attrCount;
     attrs->GetLength(&attrCount);
 
     inDOMViewNode* contentNode = nullptr;
     int32_t contentRow;
     int32_t attrRow;
-    if (mRootNode == content &&
+    if (mRootNode == el &&
         !(mWhatToShow & nsIDOMNodeFilter::SHOW_ELEMENT)) {
       // if this view has a root node but is not displaying it,
       // it is ok to act as if the changed attribute is on the root.
       attrRow = attrCount - 1;
     } else {
-      if (NS_FAILED(NodeToRow(content, &contentRow))) {
+      if (NS_FAILED(NodeToRow(el, &contentRow))) {
         return;
       }
       RowToNode(contentRow, &contentNode);
@@ -748,11 +728,11 @@ inDOMView::AttributeChanged(nsIDocument* aDocument, dom::Element* aElement,
     inDOMViewNode* contentNode = nullptr;
     int32_t contentRow;
     int32_t baseLevel;
-    if (NS_SUCCEEDED(NodeToRow(content, &contentRow))) {
+    if (NS_SUCCEEDED(NodeToRow(el, &contentRow))) {
       RowToNode(contentRow, &contentNode);
       baseLevel = contentNode->level;
     } else {
-      if (mRootNode == content) {
+      if (mRootNode == el) {
         contentRow = -1;
         baseLevel = -1;
       } else
@@ -1196,42 +1176,40 @@ nsresult
 inDOMView::GetChildNodesFor(nsIDOMNode* aNode, nsCOMArray<nsIDOMNode>& aResult)
 {
   NS_ENSURE_ARG(aNode);
-  // Need to do this test to prevent unfortunate NYI assertion
-  // on nsXULAttribute::GetChildNodes
-  nsCOMPtr<nsIDOMAttr> attr = do_QueryInterface(aNode);
-  if (!attr) {
-    // attribute nodes
-    if (mWhatToShow & nsIDOMNodeFilter::SHOW_ATTRIBUTE) {
-      nsCOMPtr<nsIDOMNamedNodeMap> attrs;
-      aNode->GetAttributes(getter_AddRefs(attrs));
+  // attribute nodes
+  if (mWhatToShow & nsIDOMNodeFilter::SHOW_ATTRIBUTE) {
+    nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aNode);
+    if (element) {
+      nsCOMPtr<nsIDOMMozNamedAttrMap> attrs;
+      element->GetAttributes(getter_AddRefs(attrs));
       if (attrs) {
         AppendAttrsToArray(attrs, aResult);
       }
     }
+  }
 
-    if (mWhatToShow & nsIDOMNodeFilter::SHOW_ELEMENT) {
-      nsCOMPtr<nsIDOMNodeList> kids;
+  if (mWhatToShow & nsIDOMNodeFilter::SHOW_ELEMENT) {
+    nsCOMPtr<nsIDOMNodeList> kids;
+    if (!mDOMUtils) {
+      mDOMUtils = do_GetService("@mozilla.org/inspector/dom-utils;1");
       if (!mDOMUtils) {
-        mDOMUtils = do_GetService("@mozilla.org/inspector/dom-utils;1");
-        if (!mDOMUtils) {
-          return NS_ERROR_FAILURE;
-        }
-      }
-
-      mDOMUtils->GetChildrenForNode(aNode, mShowAnonymous,
-                                    getter_AddRefs(kids));
-
-      if (kids) {
-        AppendKidsToArray(kids, aResult);
+        return NS_ERROR_FAILURE;
       }
     }
 
-    if (mShowSubDocuments) {
-      nsCOMPtr<nsIDOMNode> domdoc =
-        do_QueryInterface(inLayoutUtils::GetSubDocumentFor(aNode));
-      if (domdoc) {
-        aResult.AppendObject(domdoc);
-      }
+    mDOMUtils->GetChildrenForNode(aNode, mShowAnonymous,
+                                  getter_AddRefs(kids));
+
+    if (kids) {
+      AppendKidsToArray(kids, aResult);
+    }
+  }
+
+  if (mShowSubDocuments) {
+    nsCOMPtr<nsIDOMNode> domdoc =
+      do_QueryInterface(inLayoutUtils::GetSubDocumentFor(aNode));
+    if (domdoc) {
+      aResult.AppendObject(domdoc);
     }
   }
 
@@ -1296,15 +1274,15 @@ inDOMView::AppendKidsToArray(nsIDOMNodeList* aKids,
 }
 
 nsresult
-inDOMView::AppendAttrsToArray(nsIDOMNamedNodeMap* aKids,
+inDOMView::AppendAttrsToArray(nsIDOMMozNamedAttrMap* aAttributes,
                               nsCOMArray<nsIDOMNode>& aArray)
 {
   uint32_t l = 0;
-  aKids->GetLength(&l);
-  nsCOMPtr<nsIDOMNode> kid;
+  aAttributes->GetLength(&l);
+  nsCOMPtr<nsIDOMAttr> attribute;
   for (uint32_t i = 0; i < l; ++i) {
-    aKids->Item(i, getter_AddRefs(kid));
-    aArray.AppendObject(kid);
+    aAttributes->Item(i, getter_AddRefs(attribute));
+    aArray.AppendObject(attribute);
   }
   return NS_OK;
 }
