@@ -4,7 +4,7 @@
 
 package org.mozilla.gecko.sync.setup.activities;
 
-import org.mozilla.gecko.sync.Logger;
+import org.mozilla.gecko.background.common.log.Logger;
 import org.mozilla.gecko.sync.SyncConstants;
 import org.mozilla.gecko.sync.setup.InvalidSyncKeyException;
 
