@@ -11,6 +11,8 @@
 #include "nsThreadUtils.h"
 #include "mozilla/Util.h"
 #include "mozilla/dom/quota/QuotaManager.h"
+#include "mozilla/dom/quota/QuotaObject.h"
+#include "mozilla/SQLiteInterposer.h"
 
 /**
  * This preference is a workaround to allow users/sysadmins to identify
@@ -65,22 +67,35 @@ public:
    * @param id takes a telemetry histogram id. The id+1 must be an
    * equivalent histogram for the main thread. Eg, MOZ_SQLITE_OPEN_MS 
    * is followed by MOZ_SQLITE_OPEN_MAIN_THREAD_MS.
+   *
+   * @param evtFn optionally takes a function pointer to one of the
+   * SQLiteInterposer event handler functions (OnRead, OnWrite, OnFSync).
+   * Once the end TimeStamp has been determined, if the I/O occurred on the
+   * main thread then evtFn will be called with the calculated duration.
    */
-  IOThreadAutoTimer(Telemetry::ID id)
+  IOThreadAutoTimer(Telemetry::ID id,
+                    SQLiteInterposer::EventHandlerFn evtFn = nullptr)
     : start(TimeStamp::Now()),
-      id(id)
+      id(id),
+      evtFn(evtFn)
   {
   }
 
   ~IOThreadAutoTimer() {
+    TimeStamp end(TimeStamp::Now());
     uint32_t mainThread = NS_IsMainThread() ? 1 : 0;
     Telemetry::AccumulateTimeDelta(static_cast<Telemetry::ID>(id + mainThread),
-                                   start);
+                                   start, end);
+    if (evtFn && mainThread) {
+      double duration = (end - start).ToMilliseconds();
+      evtFn(duration);
+    }
   }
 
 private:
   const TimeStamp start;
   const Telemetry::ID id;
+  SQLiteInterposer::EventHandlerFn evtFn;
 };
 
 struct telemetry_file {
@@ -121,7 +136,7 @@ int
 xRead(sqlite3_file *pFile, void *zBuf, int iAmt, sqlite_int64 iOfst)
 {
   telemetry_file *p = (telemetry_file *)pFile;
-  IOThreadAutoTimer ioTimer(p->histograms->readMS);
+  IOThreadAutoTimer ioTimer(p->histograms->readMS, &SQLiteInterposer::OnRead);
   int rc;
   rc = p->pReal->pMethods->xRead(p->pReal, zBuf, iAmt, iOfst);
   // sqlite likes to read from empty files, this is normal, ignore it.
@@ -140,7 +155,7 @@ xWrite(sqlite3_file *pFile, const void *zBuf, int iAmt, sqlite_int64 iOfst)
   if (p->quotaObject && !p->quotaObject->MaybeAllocateMoreSpace(iOfst, iAmt)) {
     return SQLITE_FULL;
   }
-  IOThreadAutoTimer ioTimer(p->histograms->writeMS);
+  IOThreadAutoTimer ioTimer(p->histograms->writeMS, &SQLiteInterposer::OnWrite);
   int rc;
   rc = p->pReal->pMethods->xWrite(p->pReal, zBuf, iAmt, iOfst);
   Telemetry::Accumulate(p->histograms->writeB, rc == SQLITE_OK ? iAmt : 0);
@@ -171,7 +186,7 @@ int
 xSync(sqlite3_file *pFile, int flags)
 {
   telemetry_file *p = (telemetry_file *)pFile;
-  IOThreadAutoTimer ioTimer(p->histograms->syncMS);
+  IOThreadAutoTimer ioTimer(p->histograms->syncMS, &SQLiteInterposer::OnFSync);
   return p->pReal->pMethods->xSync(p->pReal, flags);
 }
 
@@ -289,7 +304,23 @@ xShmUnmap(sqlite3_file *pFile, int delFlag){
   rc = p->pReal->pMethods->xShmUnmap(p->pReal, delFlag);
   return rc;
 }
- 
+
+int
+xFetch(sqlite3_file *pFile, sqlite3_int64 iOff, int iAmt, void **pp)
+{
+  telemetry_file *p = (telemetry_file *)pFile;
+  MOZ_ASSERT(p->pReal->pMethods->iVersion >= 3);
+  return p->pReal->pMethods->xFetch(p->pReal, iOff, iAmt, pp);
+}
+
+int
+xUnfetch(sqlite3_file *pFile, sqlite3_int64 iOff, void *pResOut)
+{
+  telemetry_file *p = (telemetry_file *)pFile;
+  MOZ_ASSERT(p->pReal->pMethods->iVersion >= 3);
+  return p->pReal->pMethods->xUnfetch(p->pReal, iOff, pResOut);
+}
+
 int
 xOpen(sqlite3_vfs* vfs, const char *zName, sqlite3_file* pFile,
           int flags, int *pOutFlags)
@@ -336,7 +367,10 @@ xOpen(sqlite3_vfs* vfs, const char *zName, sqlite3_file* pFile,
     sqlite3_io_methods *pNew = new sqlite3_io_methods;
     const sqlite3_io_methods *pSub = p->pReal->pMethods;
     memset(pNew, 0, sizeof(*pNew));
-    pNew->iVersion = pSub->iVersion;
+    // If you update this version number, you must add appropriate IO methods
+    // for any methods added in the version change.
+    pNew->iVersion = 3;
+    MOZ_ASSERT(pNew->iVersion >= pSub->iVersion);
     pNew->xClose = xClose;
     pNew->xRead = xRead;
     pNew->xWrite = xWrite;
@@ -349,12 +383,19 @@ xOpen(sqlite3_vfs* vfs, const char *zName, sqlite3_file* pFile,
     pNew->xFileControl = xFileControl;
     pNew->xSectorSize = xSectorSize;
     pNew->xDeviceCharacteristics = xDeviceCharacteristics;
-    if( pNew->iVersion>=2 ){
-      pNew->xShmMap = pSub->xShmMap ? xShmMap : 0;
-      pNew->xShmLock = pSub->xShmLock ? xShmLock : 0;
-      pNew->xShmBarrier = pSub->xShmBarrier ? xShmBarrier : 0;
-      pNew->xShmUnmap = pSub->xShmUnmap ? xShmUnmap : 0;
-    }
+    // Methods added in version 2.
+    pNew->xShmMap = pSub->xShmMap ? xShmMap : 0;
+    pNew->xShmLock = pSub->xShmLock ? xShmLock : 0;
+    pNew->xShmBarrier = pSub->xShmBarrier ? xShmBarrier : 0;
+    pNew->xShmUnmap = pSub->xShmUnmap ? xShmUnmap : 0;
+    // Methods added in version 3.
+    // SQLite 3.7.17 calls these methods without checking for NULL first,
+    // so we always define them.  Verify that we're not going to call
+    // NULL pointers, though.
+    MOZ_ASSERT(pSub->xFetch);
+    pNew->xFetch = xFetch;
+    MOZ_ASSERT(pSub->xUnfetch);
+    pNew->xUnfetch = xUnfetch;
     pFile->pMethods = pNew;
   }
   return rc;
@@ -498,8 +539,9 @@ sqlite3_vfs* ConstructTelemetryVFS()
 
   sqlite3_vfs *tvfs = new ::sqlite3_vfs;
   memset(tvfs, 0, sizeof(::sqlite3_vfs));
+  // If you update this version number, you must add appropriate VFS methods
+  // for any methods added in the version change.
   tvfs->iVersion = 3;
-  // If the SQLite VFS version is updated, this shim must be updated as well.
   MOZ_ASSERT(vfs->iVersion == tvfs->iVersion);
   tvfs->szOsFile = sizeof(telemetry_file) - sizeof(sqlite3_file) + vfs->szOsFile;
   tvfs->mxPathname = vfs->mxPathname;
@@ -517,9 +559,9 @@ sqlite3_vfs* ConstructTelemetryVFS()
   tvfs->xSleep = xSleep;
   tvfs->xCurrentTime = xCurrentTime;
   tvfs->xGetLastError = xGetLastError;
-  // Added in version 2.
+  // Methods added in version 2.
   tvfs->xCurrentTimeInt64 = xCurrentTimeInt64;
-  // Added in version 3.
+  // Methods added in version 3.
   tvfs->xSetSystemCall = xSetSystemCall;
   tvfs->xGetSystemCall = xGetSystemCall;
   tvfs->xNextSystemCall = xNextSystemCall;
