@@ -52,6 +52,7 @@
 #include <utility>
 #include <vector>
 
+#include "common/arm_ex_reader.h"
 #include "common/dwarf/bytereader-inl.h"
 #include "common/dwarf/dwarf2diehandler.h"
 #include "common/dwarf_cfi_to_module.h"
@@ -63,9 +64,17 @@
 #include "common/linux/file_id.h"
 #include "common/module.h"
 #include "common/scoped_ptr.h"
+#ifndef NO_STABS_SUPPORT
 #include "common/stabs_reader.h"
 #include "common/stabs_to_module.h"
+#endif
 #include "common/using_std_string.h"
+#include "common/logging.h"
+
+#ifndef SHT_ARM_EXIDX
+// bionic and older glibc don't define it
+# define SHT_ARM_EXIDX (SHT_LOPROC + 1)
+#endif
 
 // This namespace contains helper functions.
 namespace {
@@ -80,7 +89,9 @@ using google_breakpad::FindElfSectionByName;
 using google_breakpad::GetOffset;
 using google_breakpad::IsValidElf;
 using google_breakpad::Module;
+#ifndef NO_STABS_SUPPORT
 using google_breakpad::StabsToModule;
+#endif
 using google_breakpad::UniqueString;
 using google_breakpad::scoped_ptr;
 
@@ -118,7 +129,7 @@ class MmapWrapper {
  public:
   MmapWrapper() : is_set_(false) {}
   ~MmapWrapper() {
-    if (base_ != NULL) {
+    if (is_set_ && base_ != NULL) {
       assert(size_ > 0);
       munmap(base_, size_);
     }
@@ -130,6 +141,7 @@ class MmapWrapper {
   }
   void release() {
     assert(is_set_);
+    is_set_ = false;
     base_ = NULL;
     size_ = 0;
   }
@@ -158,6 +170,7 @@ typename ElfClass::Addr GetLoadingAddress(
   return 0;
 }
 
+#ifndef NO_STABS_SUPPORT
 template<typename ElfClass>
 bool LoadStabs(const typename ElfClass::Ehdr* elf_header,
                const typename ElfClass::Shdr* stab_section,
@@ -183,6 +196,7 @@ bool LoadStabs(const typename ElfClass::Ehdr* elf_header,
   handler.Finalize();
   return true;
 }
+#endif  // NO_STABS_SUPPORT
 
 // A line-to-module loader that accepts line number info parsed by
 // dwarf2reader::LineInfo and populates a Module and a line vector
@@ -337,6 +351,52 @@ bool LoadDwarfCFI(const string& dwarf_filename,
   return true;
 }
 
+template<typename ElfClass>
+bool LoadARMexidx(const typename ElfClass::Ehdr* elf_header,
+                  const typename ElfClass::Shdr* exidx_section,
+                  const typename ElfClass::Shdr* extab_section,
+                  uint32_t loading_addr,
+                  Module* module) {
+  // To do this properly we need to know:
+  // * the bounds of the .ARM.exidx section in the mapped image
+  // * the bounds of the .ARM.extab section in the mapped image
+  // * the vma of the last byte in the text section associated with the .exidx
+  // The first two are easy.  The third is a bit tricky.  If we can't
+  // figure out what it is, just pass in zero.
+  const char *exidx_img
+    = GetOffset<ElfClass, char>(elf_header, exidx_section->sh_offset);
+  size_t exidx_size = exidx_section->sh_size;
+  const char *extab_img
+    = GetOffset<ElfClass, char>(elf_header, extab_section->sh_offset);
+  size_t extab_size = extab_section->sh_size;
+
+  // The sh_link field of the exidx section gives the section number
+  // for the associated text section.
+  uint32_t exidx_text_last_svma = 0;
+  int exidx_text_sno = exidx_section->sh_link;
+  typedef typename ElfClass::Shdr Shdr;
+  // |sections| points to the section header table
+  const Shdr* sections
+    = GetOffset<ElfClass, Shdr>(elf_header, elf_header->e_shoff);
+  const int num_sections = elf_header->e_shnum;
+  if (exidx_text_sno >= 0 && exidx_text_sno < num_sections) {
+    const Shdr* exidx_text_shdr = &sections[exidx_text_sno];
+    if (exidx_text_shdr->sh_size > 0) {
+      exidx_text_last_svma
+        = exidx_text_shdr->sh_addr + exidx_text_shdr->sh_size - 1;
+    }
+  }
+
+  arm_ex_to_module::ARMExToModule handler(module);
+  arm_ex_reader::ExceptionTableInfo
+    parser(exidx_img, exidx_size, extab_img, extab_size, exidx_text_last_svma,
+           &handler,
+           reinterpret_cast<const char*>(elf_header),
+           loading_addr);
+  parser.Start();
+  return true;
+}
+
 bool LoadELF(const string& obj_file, MmapWrapper* map_wrapper,
              void** elf_header) {
   int obj_fd = open(obj_file.c_str(), O_RDONLY);
@@ -418,8 +478,12 @@ string ReadDebugLink(const char* debuglink,
   }
 
   if (!found) {
-    fprintf(stderr, "Failed to open debug ELF file '%s' for '%s': %s\n",
-            debuglink_path.c_str(), obj_file.c_str(), strerror(errno));
+    fprintf(stderr, "Failed to find debug ELF file for '%s' after trying:\n",
+            obj_file.c_str());
+    for (it = debug_dirs.begin(); it < debug_dirs.end(); ++it) {
+      const string debug_dir = *it;
+      fprintf(stderr, "  %s/%s\n", debug_dir.c_str(), debuglink);
+    }
     return "";
   }
 
@@ -517,6 +581,9 @@ bool LoadSymbols(const string& obj_file,
   typedef typename ElfClass::Phdr Phdr;
   typedef typename ElfClass::Shdr Shdr;
 
+  BPLOG(INFO) << "";
+  BPLOG(INFO) << "LoadSymbols: BEGIN   " << obj_file;
+
   Addr loading_addr = GetLoadingAddress<ElfClass>(
       GetOffset<ElfClass, Phdr>(elf_header, elf_header->e_phoff),
       elf_header->e_phnum);
@@ -533,6 +600,7 @@ bool LoadSymbols(const string& obj_file,
   bool found_usable_info = false;
 
   if (symbol_data != ONLY_CFI) {
+#ifndef NO_STABS_SUPPORT
     // Look for STABS debugging information, and load it if present.
     const Shdr* stab_section =
       FindElfSectionByName<ElfClass>(".stab", SHT_PROGBITS,
@@ -551,6 +619,7 @@ bool LoadSymbols(const string& obj_file,
         }
       }
     }
+#endif  // NO_STABS_SUPPORT
 
     // Look for DWARF debugging information, and load it if present.
     const Shdr* dwarf_section =
@@ -584,6 +653,8 @@ bool LoadSymbols(const string& obj_file,
                                  dwarf_cfi_section, false, 0, 0, big_endian,
                                  module);
       found_usable_info = found_usable_info || result;
+      if (result)
+        BPLOG(INFO) << "LoadSymbols:   read CFI from .debug_frame";
     }
 
     // Linux C++ exception handling information can also provide
@@ -610,10 +681,35 @@ bool LoadSymbols(const string& obj_file,
                                  eh_frame_section, true,
                                  got_section, text_section, big_endian, module);
       found_usable_info = found_usable_info || result;
+      if (result)
+        BPLOG(INFO) << "LoadSymbols:   read CFI from .eh_frame";
     }
   }
 
-  if (!found_debug_info_section) {
+  // ARM has special unwind tables that can be used.
+  const Shdr* arm_exidx_section =
+      FindElfSectionByName<ElfClass>(".ARM.exidx", SHT_ARM_EXIDX,
+                                     sections, names, names_end,
+                                     elf_header->e_shnum);
+  const Shdr* arm_extab_section =
+      FindElfSectionByName<ElfClass>(".ARM.extab", SHT_PROGBITS,
+                                     sections, names, names_end,
+                                     elf_header->e_shnum);
+  // Only load information from this section if there isn't a .debug_info
+  // section.
+  if (!found_debug_info_section
+      && arm_exidx_section && arm_extab_section && symbol_data != NO_CFI) {
+    info->LoadedSection(".ARM.exidx");
+    info->LoadedSection(".ARM.extab");
+    bool result = LoadARMexidx<ElfClass>(elf_header,
+                                         arm_exidx_section, arm_extab_section,
+                                         loading_addr, module);
+    found_usable_info = found_usable_info || result;
+    if (result)
+      BPLOG(INFO) << "LoadSymbols:   read EXIDX from .ARM.{exidx,extab}";
+  }
+
+  if (!found_debug_info_section && symbol_data != ONLY_CFI) {
     fprintf(stderr, "%s: file contains no debugging information"
             " (no \".stab\" or \".debug_info\" sections)\n",
             obj_file.c_str());
@@ -642,45 +738,54 @@ bool LoadSymbols(const string& obj_file,
         fprintf(stderr, "%s does not contain a .gnu_debuglink section.\n",
                 obj_file.c_str());
       }
-    } else if (symbol_data != ONLY_CFI) {
-      // The caller doesn't want to consult .gnu_debuglink.
-      // See if there are export symbols available.
-      const Shdr* dynsym_section =
+    } else {
+      if (symbol_data != ONLY_CFI) {
+        // The caller doesn't want to consult .gnu_debuglink.
+        // See if there are export symbols available.
+        const Shdr* dynsym_section =
           FindElfSectionByName<ElfClass>(".dynsym", SHT_DYNSYM,
                                          sections, names, names_end,
                                          elf_header->e_shnum);
-      const Shdr* dynstr_section =
+        const Shdr* dynstr_section =
           FindElfSectionByName<ElfClass>(".dynstr", SHT_STRTAB,
                                          sections, names, names_end,
                                          elf_header->e_shnum);
-      if (dynsym_section && dynstr_section) {
-        info->LoadedSection(".dynsym");
+        if (dynsym_section && dynstr_section) {
+          info->LoadedSection(".dynsym");
 
-        const uint8_t* dynsyms =
-            GetOffset<ElfClass, uint8_t>(elf_header, dynsym_section->sh_offset);
-        const uint8_t* dynstrs =
-            GetOffset<ElfClass, uint8_t>(elf_header, dynstr_section->sh_offset);
-        bool result =
-            ELFSymbolsToModule(dynsyms,
-                               dynsym_section->sh_size,
-                               dynstrs,
-                               dynstr_section->sh_size,
-                               big_endian,
-                               ElfClass::kAddrSize,
-                               module);
-        found_usable_info = found_usable_info || result;
+          const uint8_t* dynsyms =
+              GetOffset<ElfClass, uint8_t>(elf_header,
+                                           dynsym_section->sh_offset);
+          const uint8_t* dynstrs =
+              GetOffset<ElfClass, uint8_t>(elf_header,
+                                           dynstr_section->sh_offset);
+          bool result =
+              ELFSymbolsToModule(dynsyms,
+                                 dynsym_section->sh_size,
+                                 dynstrs,
+                                 dynstr_section->sh_size,
+                                 big_endian,
+                                 ElfClass::kAddrSize,
+                                 module);
+          found_usable_info = found_usable_info || result;
+        }
       }
 
       // Return true if some usable information was found, since
       // the caller doesn't want to use .gnu_debuglink.
+      BPLOG(INFO) << "LoadSymbols: " 
+                  << (found_usable_info ? "SUCCESS " : "FAILURE ")
+                  << obj_file;
       return found_usable_info;
     }
 
     // No debug info was found, let the user try again with .gnu_debuglink
     // if present.
+    BPLOG(INFO) << "LoadSymbols: FAILURE " << obj_file;
     return false;
   }
 
+  BPLOG(INFO) << "LoadSymbols: SUCCESS " << obj_file;
   return true;
 }
 
@@ -716,7 +821,7 @@ string FormatIdentifier(unsigned char identifier[16]) {
   for (int i = 0; identifier_str[i] != '\0'; ++i)
     if (identifier_str[i] != '-')
       id_no_dash += identifier_str[i];
-  // Add an extra "0" by the end.  PDB files on Windows have an 'age'
+  // Add an extra "0" by the end.  PDB files on Windows.have an 'age'
   // number appended to the end of the file identifier; this isn't
   // really used or necessary on other platforms, but be consistent.
   id_no_dash += '0';
